"""
This type stub file was generated by pyright.
"""

"""Infrastructure for registering and firing callbacks on application events.

Unlike :mod:`IPython.core.hooks`, which lets end users set single functions to
be called at specific times, or a collection of alternative methods to try,
callbacks are designed to be used by extension authors. A number of callbacks
can be registered for the same event without needing to be aware of one another.

The functions defined in this module are no-ops indicating the names of available
events and the arguments which will be passed to them.

.. note::

   This API is experimental in IPython 2.0, and may be revised in future versions.
"""
class EventManager:
    """Manage a collection of events and a sequence of callbacks for each.
    
    This is attached to :class:`~IPython.core.interactiveshell.InteractiveShell`
    instances as an ``events`` attribute.
    
    .. note::

       This API is experimental in IPython 2.0, and may be revised in future versions.
    """
    def __init__(self, shell, available_events) -> None:
        """Initialise the :class:`CallbackManager`.

        Parameters
        ----------
        shell
            The :class:`~IPython.core.interactiveshell.InteractiveShell` instance
        available_events
            An iterable of names for callback events.
        """
        ...
    
    def register(self, event, function): # -> None:
        """Register a new event callback.

        Parameters
        ----------
        event : str
            The event for which to register this callback.
        function : callable
            A function to be called on the given event. It should take the same
            parameters as the appropriate callback prototype.

        Raises
        ------
        TypeError
            If ``function`` is not callable.
        KeyError
            If ``event`` is not one of the known events.
        """
        ...
    
    def unregister(self, event, function):
        """Remove a callback from the given event."""
        ...
    
    def trigger(self, event, *args, **kwargs): # -> None:
        """Call callbacks for ``event``.

        Any additional arguments are passed to all callbacks registered for this
        event. Exceptions raised by callbacks are caught, and a message printed.
        """
        ...
    


available_events = ...
@_define_event
def pre_execute(): # -> None:
    """Fires before code is executed in response to user/frontend action.

    This includes comm and widget messages and silent execution, as well as user
    code cells.
    """
    ...

@_define_event
def pre_run_cell(info): # -> None:
    """Fires before user-entered code runs.

    Parameters
    ----------
    info : :class:`~IPython.core.interactiveshell.ExecutionInfo`
        An object containing information used for the code execution.
    """
    ...

@_define_event
def post_execute(): # -> None:
    """Fires after code is executed in response to user/frontend action.

    This includes comm and widget messages and silent execution, as well as user
    code cells.
    """
    ...

@_define_event
def post_run_cell(result): # -> None:
    """Fires after user-entered code runs.

    Parameters
    ----------
    result : :class:`~IPython.core.interactiveshell.ExecutionResult`
        The object which will be returned as the execution result.
    """
    ...

@_define_event
def shell_initialized(ip): # -> None:
    """Fires after initialisation of :class:`~IPython.core.interactiveshell.InteractiveShell`.

    This is before extensions and startup scripts are loaded, so it can only be
    set by subclassing.

    Parameters
    ----------
    ip : :class:`~IPython.core.interactiveshell.InteractiveShell`
        The newly initialised shell.
    """
    ...




from typing import Sequence, TypeVar
from numpy.random import RandomState
from ...base import BaseEstimator, TransformerMixin
from .common import (
    X_DTYPE as X_DTYPE,
    X_BINNED_DTYPE as X_BINNED_DTYPE,
    ALMOST_INF as ALMOST_INF,
    X_BITSET_INNER_DTYPE as X_BITSET_INNER_DTYPE,
)
from numpy import ndarray, uint8
from ._bitset import set_bitset_memoryview as set_bitset_memoryview
from ...utils.validation import check_is_fitted as check_is_fitted
from ...utils import (
    check_random_state as check_random_state,
    check_array as check_array,
)
from ..._typing import Int, ArrayLike, MatrixLike
from ...utils.fixes import percentile as percentile

_BinMapper_Self = TypeVar("_BinMapper_Self", bound="_BinMapper")

# Author: Nicolas Hug

import numpy as np


class _BinMapper(TransformerMixin, BaseEstimator):
    missing_values_bin_idx_: uint8 = ...
    is_categorical_: ndarray = ...
    n_bins_non_missing_: ndarray = ...
    bin_thresholds_: list[ndarray] = ...

    def __init__(
        self,
        n_bins: Int = 256,
        subsample: None | int = ...,
        is_categorical: None | ArrayLike = None,
        known_categories: Sequence[None | ArrayLike] | None | ArrayLike = None,
        random_state: RandomState | None | Int = None,
        n_threads: None | Int = None,
    ) -> None:
        ...

    def fit(self: _BinMapper_Self, X: MatrixLike, y=None) -> _BinMapper_Self:
        ...

    def transform(self, X: MatrixLike) -> ndarray:
        ...

    def make_known_categories_bitsets(self) -> tuple[ndarray, ndarray]:
        ...



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014, Nicolas P. Rougier
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------
import re
import math

from .base import units
from .. import logger

class Length(object):
    def __init__(self, content, mode="x", parent=None): ...
    def __float__(self): ...
    @property
    def value(self): ...
    def __repr__(self): ...

class XLength(Length):
    def __init__(self, content, parent=None): ...

class YLength(Length):
    def __init__(self, content, parent=None): ...

class XYLength(Length):
    def __init__(self, content, parent=None): ...



"""
This type stub file was generated by pyright.
"""

import unittest
from contextlib import contextmanager

"""Generic testing tools.

Authors
-------
- Fernando Perez <Fernando.Perez@berkeley.edu>
"""
doctest_deco = ...
@doctest_deco
def full_path(startPath, files):
    """Make full paths for all the listed files, based on startPath.

    Only the base part of startPath is kept, since this routine is typically
    used with a script's ``__file__`` variable as startPath. The base of startPath
    is then prepended to all the listed files, forming the output list.

    Parameters
    ----------
    startPath : string
      Initial path to use as the base for the results.  This path is split
      using os.path.split() and only its first component is kept.

    files : string or list
      One or more files.

    Examples
    --------

    >>> full_path('/foo/bar.py',['a.txt','b.txt'])
    ['/foo/a.txt', '/foo/b.txt']

    >>> full_path('/foo',['a.txt','b.txt'])
    ['/a.txt', '/b.txt']

    If a single file is given, the output is still a list::

        >>> full_path('/foo','a.txt')
        ['/a.txt']
    """
    ...

def parse_test_output(txt):
    """Parse the output of a test run and return errors, failures.

    Parameters
    ----------
    txt : str
      Text output of a test run, assumed to contain a line of one of the
      following forms::

        'FAILED (errors=1)'
        'FAILED (failures=1)'
        'FAILED (errors=1, failures=1)'

    Returns
    -------
    nerr, nfail
      number of errors and failures.
    """
    ...

def default_argv():
    """Return a valid default argv for creating testing instances of ipython"""
    ...

def default_config(): # -> Config:
    """Return a config object with good defaults for testing."""
    ...

def get_ipython_cmd(as_string=...):
    """
    Return appropriate IPython command line name. By default, this will return
    a list that can be used with subprocess.Popen, for example, but passing
    `as_string=True` allows for returning the IPython command as a string.

    Parameters
    ----------
    as_string: bool
        Flag to allow to return the command as a string.
    """
    ...

def ipexec(fname, options=..., commands=...):
    """Utility to call 'ipython filename'.

    Starts IPython with a minimal and safe configuration to make startup as fast
    as possible.

    Note that this starts IPython in a subprocess!

    Parameters
    ----------
    fname : str, Path
      Name of file to be executed (should have .py or .ipy extension).

    options : optional, list
      Extra command-line flags to be passed to IPython.

    commands : optional, list
      Commands to send in on stdin

    Returns
    -------
    ``(stdout, stderr)`` of ipython subprocess.
    """
    ...

def ipexec_validate(fname, expected_out, expected_err=..., options=..., commands=...): # -> None:
    """Utility to call 'ipython filename' and validate output/error.

    This function raises an AssertionError if the validation fails.

    Note that this starts IPython in a subprocess!

    Parameters
    ----------
    fname : str, Path
      Name of the file to be executed (should have .py or .ipy extension).

    expected_out : str
      Expected stdout of the process.

    expected_err : optional, str
      Expected stderr of the process.

    options : optional, list
      Extra command-line flags to be passed to IPython.

    Returns
    -------
    None
    """
    ...

class TempFileMixin(unittest.TestCase):
    """Utility class to create temporary Python/IPython files.

    Meant as a mixin class for test cases."""
    def mktmp(self, src, ext=...): # -> None:
        """Make a valid python temp file."""
        ...
    
    def tearDown(self): # -> None:
        ...
    
    def __enter__(self): # -> Self@TempFileMixin:
        ...
    
    def __exit__(self, exc_type, exc_value, traceback): # -> None:
        ...
    


pair_fail_msg = ...
def check_pairs(func, pairs): # -> None:
    """Utility function for the common case of checking a function with a
    sequence of input/output pairs.

    Parameters
    ----------
    func : callable
      The function to be tested. Should accept a single argument.
    pairs : iterable
      A list of (input, expected_output) tuples.

    Returns
    -------
    None. Raises an AssertionError if any output does not match the expected
    value.
    """
    ...

MyStringIO = ...
_re_type = ...
notprinted_msg = ...
class AssertPrints:
    """Context manager for testing that code prints certain text.

    Examples
    --------
    >>> with AssertPrints("abc", suppress=False):
    ...     print("abcd")
    ...     print("def")
    ...
    abcd
    def
    """
    def __init__(self, s, channel=..., suppress=...) -> None:
        ...
    
    def __enter__(self): # -> None:
        ...
    
    def __exit__(self, etype, value, traceback):
        ...
    


printed_msg = ...
class AssertNotPrints(AssertPrints):
    """Context manager for checking that certain output *isn't* produced.

    Counterpart of AssertPrints"""
    def __exit__(self, etype, value, traceback):
        ...
    


@contextmanager
def mute_warn():
    ...

@contextmanager
def make_tempfile(name):
    """Create an empty, named, temporary file for the duration of the context."""
    ...

def fake_input(inputs): # -> _patch:
    """Temporarily replace the input() function to return the given values

    Use as a context manager:

    with fake_input(['result1', 'result2']):
        ...

    Values are returned in order. If input() is called again after the last value
    was used, EOFError is raised.
    """
    ...

def help_output_test(subcommand=...):
    """test that `ipython [subcommand] -h` works"""
    ...

def help_all_output_test(subcommand=...):
    """test that `ipython [subcommand] --help-all` works"""
    ...




from matplotlib.path import Path
import numpy as np
from ._typing import *
from typing import Callable

class NonIntersectingPathException(ValueError): ...

def get_intersection(cx1, cy1, cos_t1, sin_t1, cx2, cy2, cos_t2, sin_t2)-> tuple[float, float]: ...
def get_normal_points(cx, cy, cos_t, sin_t, length)-> tuple[float, float, float, float]: ...
def split_de_casteljau(beta, t)-> float: ...
def find_bezier_t_intersecting_with_closedpath(
    bezier_point_at_t: Callable,
    inside_closedpath: Callable,
    t0: float = ...,
    t1: float = ...,
    tolerance: float = ...,
) -> tuple[float, float]: ...

class BezierSegment:
    def __init__(self, control_points) -> None: ...
    def __call__(self, t: ArrayLike) -> tuple: ...
    def point_at_t(self, t) -> tuple[float]: ...
    @property
    def control_points(self)-> np.ndarray: ...
    @property
    def dimension(self)-> int: ...
    @property
    def degree(self) -> int: ...
    @property
    def polynomial_coefficients(self): ...
    def axis_aligned_extrema(self) -> tuple: ...

def split_bezier_intersecting_with_closedpath(
    bezier, inside_closedpath: Callable, tolerance: float = ...
) -> tuple[list, list]: ...
def split_path_inout(path, inside, tolerance=..., reorder_inout=...)-> tuple[Path, Path]: ...
def inside_circle(cx, cy, r) -> Callable: ...
def get_cos_sin(x0, y0, x1, y1)-> tuple[float, float]: ...
def check_if_parallel(
    dx1: float, dy1: float, dx2: float, dy2: float, tolerance: float = ...
) -> bool: ...
def get_parallels(bezier2, width)-> tuple[list[tuple[float, float]], list[tuple[float, float]]]: ...
def find_control_points(c1x, c1y, mmx, mmy, c2x, c2y)-> list[tuple[float, float]]: ...
def make_wedged_bezier2(bezier2, width, w1=..., wm=..., w2=...)-> tuple[list[tuple[float, float]], list[tuple[float, float]]]: ...



from numpy.typing import ArrayLike
from collections import defaultdict, deque
from itertools import chain, combinations, islice

from ..classes.graph import Graph
from ..utils import not_implemented_for

__all__ = [
    "find_cliques",
    "find_cliques_recursive",
    "make_max_clique_graph",
    "make_clique_bipartite",
    "graph_clique_number",
    "graph_number_of_cliques",
    "node_clique_number",
    "number_of_cliques",
    "cliques_containing_node",
    "enumerate_all_cliques",
    "max_weight_clique",
]

def enumerate_all_cliques(G: Graph): ...
def find_cliques(G: Graph, nodes=None): ...

# TODO Should this also be not implemented for directed graphs?
def find_cliques_recursive(G: Graph, nodes=None): ...
def make_max_clique_graph(G: Graph, create_using=None): ...
def make_clique_bipartite(
    G: Graph, fpos: bool | None = None, create_using=None, name=None
): ...
def graph_clique_number(G: Graph, cliques: ArrayLike | None = None) -> int: ...
def graph_number_of_cliques(G: Graph, cliques: ArrayLike | None = None) -> int: ...
def node_clique_number(
    G: Graph, nodes=None, cliques=None, separate_nodes=False
) -> int | dict: ...
def number_of_cliques(G: Graph, nodes=None, cliques=None): ...
def cliques_containing_node(G: Graph, nodes=None, cliques=None): ...

class MaxWeightClique:
    def __init__(self, G: Graph, weight): ...
    def update_incumbent_if_improved(self, C, C_weight): ...
    def greedily_find_independent_set(self, P): ...
    def find_branching_nodes(self, P, target): ...
    def expand(self, C, C_weight, P): ...
    def find_max_weight_clique(self): ...

def max_weight_clique(G: Graph, weight="weight") -> tuple[ArrayLike, int]: ...



from typing import Any, Callable, ClassVar, TypeVar
from numpy.random import RandomState
from ._base import LinearRegression as LinearRegression
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from ..utils._param_validation import (
    Interval as Interval,
    Options as Options,
    StrOptions as StrOptions,
    HasMethods as HasMethods,
    Hidden as Hidden,
)
from numpy import ndarray
from numbers import Integral as Integral, Real as Real
from ..base import (
    BaseEstimator,
    MetaEstimatorMixin,
    RegressorMixin,
    clone as clone,
    MultiOutputMixin,
)
from ..utils.random import sample_without_replacement as sample_without_replacement
from .._typing import Float, Int, MatrixLike, ArrayLike
from ..utils import (
    check_random_state as check_random_state,
    check_consistent_length as check_consistent_length,
)
from ..utils.validation import (
    check_is_fitted as check_is_fitted,
    has_fit_parameter as has_fit_parameter,
)

RANSACRegressor_Self = TypeVar("RANSACRegressor_Self", bound="RANSACRegressor")

# Author: Johannes Schönberger
#
# License: BSD 3 clause

import warnings

import numpy as np

_EPSILON = ...


class RANSACRegressor(
    MetaEstimatorMixin, RegressorMixin, MultiOutputMixin, BaseEstimator
):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_skips_invalid_model_: int = ...
    n_skips_invalid_data_: int = ...
    n_skips_no_inliers_: int = ...
    inlier_mask_: ndarray = ...
    n_trials_: int = ...
    estimator_: Any = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        estimator: Any = None,
        *,
        min_samples: float | None | int = None,
        residual_threshold: None | Float = None,
        is_data_valid: None | Callable = None,
        is_model_valid: None | Callable = None,
        max_trials: Int = 100,
        max_skips: Int = ...,
        stop_n_inliers: Int = ...,
        stop_score: Float = ...,
        stop_probability: float = 0.99,
        loss: str | Callable = "absolute_error",
        random_state: RandomState | None | Int = None,
        base_estimator: Any = "deprecated",
    ) -> None:
        ...

    def fit(
        self: RANSACRegressor_Self,
        X: MatrixLike | ArrayLike,
        y: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> RANSACRegressor_Self:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...

    def score(self, X: MatrixLike, y: MatrixLike | ArrayLike) -> float:
        ...



import heapq

__all__ = ["MappedQueue"]

class _HeapElement:

    __slots__: list = ...

    def __init__(self, priority, element): ...
    def __lt__(self, other): ...
    def __gt__(self, other): ...
    def __eq__(self, other): ...
    def __hash__(self): ...
    def __getitem__(self, indx): ...
    def __iter__(self): ...
    def __repr__(self): ...

class MappedQueue:
    def __init__(self, data=[]): ...
    def _heapify(self): ...
    def __len__(self): ...
    def push(self, elt, priority=None): ...
    def pop(self): ...
    def update(self, elt, new, priority=None): ...
    def remove(self, elt): ...
    def _siftup(self, pos): ...
    def _siftdown(self, start_pos, pos): ...



from typing import Any
from ..classes.reportviews import NodeDataView
from collections.abc import Mapping, Set

from ..classes.graph import Graph

__all__ = [
    "NodeView",
    "NodeDataView",
    "EdgeView",
    "OutEdgeView",
    "InEdgeView",
    "EdgeDataView",
    "OutEdgeDataView",
    "InEdgeDataView",
    "MultiEdgeView",
    "OutMultiEdgeView",
    "InMultiEdgeView",
    "MultiEdgeDataView",
    "OutMultiEdgeDataView",
    "InMultiEdgeDataView",
    "DegreeView",
    "DiDegreeView",
    "InDegreeView",
    "OutDegreeView",
    "MultiDegreeView",
    "DiMultiDegreeView",
    "InMultiDegreeView",
    "OutMultiDegreeView",
]

# NodeViews
class NodeView(Mapping, Set):

    __slots__ = ...

    def __getstate__(self): ...
    def __setstate__(self, state): ...
    def __init__(self, graph): ...

    # Mapping methods
    def __len__(self): ...
    def __iter__(self): ...
    def __getitem__(self, n): ...

    # Set methods
    def __contains__(self, n): ...
    @classmethod
    def _from_iterable(cls, it): ...

    # DataView method
    def __call__(self, data=False, default=None): ...
    def data(self, data=True, default: Any = None) -> NodeDataView: ...
    def __str__(self): ...
    def __repr__(self): ...

class NodeDataView(Set):

    __slots__ = ...

    def __getstate__(self): ...
    def __setstate__(self, state): ...
    def __init__(self, nodedict, data=False, default=None): ...
    @classmethod
    def _from_iterable(cls, it): ...
    def __len__(self): ...
    def __iter__(self): ...
    def __contains__(self, n): ...
    def __getitem__(self, n): ...
    def __str__(self): ...
    def __repr__(self): ...

# DegreeViews
class DiDegreeView:
    def __init__(self, G: Graph, nbunch=None, weight=None): ...
    def __call__(self, nbunch=None, weight=None): ...
    def __getitem__(self, n): ...
    def __iter__(self): ...
    def __len__(self): ...
    def __str__(self): ...
    def __repr__(self): ...

class DegreeView(DiDegreeView):
    def __getitem__(self, n): ...
    def __iter__(self): ...

class OutDegreeView(DiDegreeView):
    def __getitem__(self, n): ...
    def __iter__(self): ...

class InDegreeView(DiDegreeView):
    def __getitem__(self, n): ...
    def __iter__(self): ...

class MultiDegreeView(DiDegreeView):
    def __getitem__(self, n): ...
    def __iter__(self): ...

class DiMultiDegreeView(DiDegreeView):
    def __getitem__(self, n): ...
    def __iter__(self): ...

class InMultiDegreeView(DiDegreeView):
    def __getitem__(self, n): ...
    def __iter__(self): ...

class OutMultiDegreeView(DiDegreeView):
    def __getitem__(self, n): ...
    def __iter__(self): ...

# EdgeDataViews
class OutEdgeDataView:

    __slots__ = ...

    def __getstate__(self): ...
    def __setstate__(self, state): ...
    def __init__(self, viewer, nbunch=None, data=False, default=None): ...
    def __len__(self): ...
    def __iter__(self): ...
    def __contains__(self, e): ...
    def __str__(self): ...
    def __repr__(self): ...

class EdgeDataView(OutEdgeDataView):

    __slots__ = ...

    def __len__(self): ...
    def __iter__(self): ...
    def __contains__(self, e): ...

class InEdgeDataView(OutEdgeDataView):

    __slots__ = ...

    def __iter__(self): ...
    def __contains__(self, e): ...

class OutMultiEdgeDataView(OutEdgeDataView):

    __slots__ = ...

    def __getstate__(self): ...
    def __setstate__(self, state): ...
    def __init__(self, viewer, nbunch=None, data=False, keys=False, default=None): ...
    def __len__(self): ...
    def __iter__(self): ...
    def __contains__(self, e): ...

class MultiEdgeDataView(OutMultiEdgeDataView):

    __slots__ = ...

    def __iter__(self): ...
    def __contains__(self, e): ...

class InMultiEdgeDataView(OutMultiEdgeDataView):

    __slots__ = ...

    def __iter__(self): ...
    def __contains__(self, e): ...

# EdgeViews    have set operations and no data reported
class OutEdgeView(Set, Mapping):

    __slots__ = ...

    def __getstate__(self): ...
    def __setstate__(self, state): ...
    @classmethod
    def _from_iterable(cls, it): ...

    dataview = ...

    def __init__(self, G): ...

    # Set methods
    def __len__(self): ...
    def __iter__(self): ...
    def __contains__(self, e): ...

    # Mapping Methods
    def __getitem__(self, e): ...

    # EdgeDataView methods
    def __call__(self, nbunch=None, data=False, default=None): ...
    def data(self, data=True, default: Any = None, nbunch=None): ...

    # String Methods
    def __str__(self): ...
    def __repr__(self): ...

class EdgeView(OutEdgeView):

    __slots__ = ...

    dataview = ...

    def __len__(self): ...
    def __iter__(self): ...
    def __contains__(self, e): ...

class InEdgeView(OutEdgeView):

    __slots__ = ...

    def __setstate__(self, state): ...

    dataview = ...

    def __init__(self, G): ...
    def __iter__(self): ...
    def __contains__(self, e): ...
    def __getitem__(self, e): ...

class OutMultiEdgeView(OutEdgeView):

    __slots__ = ...

    dataview = ...

    def __len__(self): ...
    def __iter__(self): ...
    def __contains__(self, e): ...
    def __getitem__(self, e): ...
    def __call__(self, nbunch=None, data=False, keys=False, default=None): ...
    def data(self, data=True, keys=False, default: Any = None, nbunch=None): ...

class MultiEdgeView(OutMultiEdgeView):

    __slots__ = ...

    dataview = ...

    def __len__(self): ...
    def __iter__(self): ...

class InMultiEdgeView(OutMultiEdgeView):

    __slots__ = ...

    def __setstate__(self, state): ...

    dataview = ...

    def __init__(self, G): ...
    def __iter__(self): ...
    def __contains__(self, e): ...
    def __getitem__(self, e): ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from ._color_dict import (
    get_color_names as get_color_names,
    get_color_dict as get_color_dict,
)  # noqa
from .color_array import Color as Color, ColorArray as ColorArray
from .colormap import (
    Colormap as Colormap,
    BaseColormap as BaseColormap,  # noqa
    get_colormap as get_colormap,
    get_colormaps as get_colormaps,
)  # noqa

__all__ = [
    "Color",
    "ColorArray",
    "Colormap",
    "BaseColormap",
    "get_colormap",
    "get_colormaps",
    "get_color_names",
    "get_color_dict",
]



import pickle as pkl

def load_reduce(self) -> None: ...
def load_newobj(self) -> None: ...
def load_newobj_ex(self) -> None: ...
def load(fh, encoding: str | None = ..., is_verbose: bool = ...): ...



"""
This type stub file was generated by pyright.
"""

import importlib
import os

aliases = ...
backends = ...
registered = ...
def register(name, inputhook): # -> None:
    """Register the function *inputhook* as an event loop integration."""
    ...

class UnknownBackend(KeyError):
    def __init__(self, name) -> None:
        ...
    
    def __str__(self) -> str:
        ...
    


def set_qt_api(gui): # -> None:
    """Sets the `QT_API` environment variable if it isn't already set."""
    ...

def get_inputhook_name_and_func(gui):
    ...




from typing import ClassVar, Sequence, TypeVar
from .model_selection import cross_val_predict as cross_val_predict
from .ensemble._forest import RandomForestRegressor
from abc import ABCMeta, abstractmethod
from .base import (
    clone as clone,
    MetaEstimatorMixin,
    RegressorMixin,
    ClassifierMixin,
    is_classifier as is_classifier,
)
from .utils.validation import (
    check_is_fitted as check_is_fitted,
    has_fit_parameter as has_fit_parameter,
)
from .base import BaseEstimator
from .utils.multiclass import (
    check_classification_targets as check_classification_targets,
)
from numpy import ndarray
from numbers import Integral as Integral
from .utils.metaestimators import available_if as available_if
from .utils import check_random_state as check_random_state
from .utils._param_validation import HasMethods as HasMethods, StrOptions as StrOptions
from ._typing import MatrixLike, ArrayLike
from scipy.sparse import spmatrix
from .utils.parallel import delayed as delayed, Parallel as Parallel
from .linear_model._logistic import LogisticRegression

MultiOutputClassifier_Self = TypeVar(
    "MultiOutputClassifier_Self", bound="MultiOutputClassifier"
)
RegressorChain_Self = TypeVar("RegressorChain_Self", bound="RegressorChain")
_BaseChain_Self = TypeVar("_BaseChain_Self", bound="_BaseChain")
_MultiOutputEstimator_Self = TypeVar(
    "_MultiOutputEstimator_Self", bound="_MultiOutputEstimator"
)
MultiOutputRegressor_Self = TypeVar(
    "MultiOutputRegressor_Self", bound="MultiOutputRegressor"
)
ClassifierChain_Self = TypeVar("ClassifierChain_Self", bound="ClassifierChain")


import numpy as np
import scipy.sparse as sp

__all__ = [
    "MultiOutputRegressor",
    "MultiOutputClassifier",
    "ClassifierChain",
    "RegressorChain",
]


class _MultiOutputEstimator(MetaEstimatorMixin, BaseEstimator, metaclass=ABCMeta):

    _parameter_constraints: ClassVar[dict] = ...

    @abstractmethod
    def __init__(self, estimator: RandomForestRegressor, *, n_jobs=None) -> None:
        ...

    def partial_fit(
        self: _MultiOutputEstimator_Self,
        X: MatrixLike | ArrayLike,
        y: MatrixLike,
        classes: Sequence[ArrayLike] | None = None,
        sample_weight: None | ArrayLike = None,
    ) -> _MultiOutputEstimator_Self:
        ...

    def fit(
        self: _MultiOutputEstimator_Self,
        X: MatrixLike | ArrayLike,
        y: MatrixLike,
        sample_weight: None | ArrayLike = None,
        **fit_params
    ) -> _MultiOutputEstimator_Self | MultiOutputRegressor:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray | spmatrix:
        ...


class MultiOutputRegressor(RegressorMixin, _MultiOutputEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    estimators_: list[BaseEstimator] = ...

    def __init__(
        self,
        estimator: BaseEstimator | RandomForestRegressor,
        *,
        n_jobs: None | int = None
    ) -> None:
        ...

    def partial_fit(
        self: MultiOutputRegressor_Self,
        X: MatrixLike | ArrayLike,
        y: MatrixLike,
        sample_weight: None | ArrayLike = None,
    ) -> MultiOutputRegressor_Self:
        ...


class MultiOutputClassifier(ClassifierMixin, _MultiOutputEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    estimators_: list[BaseEstimator] = ...
    classes_: ndarray = ...

    def __init__(self, estimator: BaseEstimator, *, n_jobs: None | int = None) -> None:
        ...

    def fit(
        self: MultiOutputClassifier_Self,
        X: MatrixLike | ArrayLike,
        Y: MatrixLike,
        sample_weight: None | ArrayLike = None,
        **fit_params
    ) -> MultiOutputClassifier_Self:
        ...

    def predict_proba(self, X: MatrixLike) -> ndarray | list[ndarray]:
        ...

    def score(self, X: MatrixLike, y: MatrixLike) -> float:
        ...


class _BaseChain(BaseEstimator, metaclass=ABCMeta):

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        base_estimator: LogisticRegression,
        *,
        order=None,
        cv=None,
        random_state=None,
        verbose: bool = False
    ) -> None:
        ...

    @abstractmethod
    def fit(
        self: _BaseChain_Self, X: MatrixLike | ArrayLike, Y: MatrixLike, **fit_params
    ) -> _BaseChain_Self | ClassifierChain:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...


class ClassifierChain(MetaEstimatorMixin, ClassifierMixin, _BaseChain):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    order_: list = ...
    estimators_: list = ...
    classes_: list = ...

    def fit(
        self: ClassifierChain_Self, X: MatrixLike | ArrayLike, Y: MatrixLike
    ) -> ClassifierChain_Self:
        ...

    def predict_proba(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def decision_function(self, X: MatrixLike) -> ndarray:
        ...


class RegressorChain(MetaEstimatorMixin, RegressorMixin, _BaseChain):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    order_: list = ...
    estimators_: list = ...

    def fit(
        self: RegressorChain_Self,
        X: MatrixLike | ArrayLike,
        Y: MatrixLike,
        **fit_params
    ) -> RegressorChain_Self:
        ...



from typing import Any, ClassVar, Literal, TypeVar
from numpy.random import RandomState
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numbers import Integral as Integral, Real as Real
from .._config import config_context as config_context
from ..metrics import (
    euclidean_distances as euclidean_distances,
    pairwise_distances_argmin as pairwise_distances_argmin,
)
from ..base import BaseEstimator, ClusterMixin
from .._typing import MatrixLike, ArrayLike, Int, Float
from ..utils import (
    as_float_array as as_float_array,
    check_random_state as check_random_state,
)
from ..utils.validation import check_is_fitted as check_is_fitted

AffinityPropagation_Self = TypeVar(
    "AffinityPropagation_Self", bound="AffinityPropagation"
)

import warnings

import numpy as np


###############################################################################
# Public API


def affinity_propagation(
    S: MatrixLike,
    *,
    preference: float | None | ArrayLike = None,
    convergence_iter: Int = 15,
    max_iter: Int = 200,
    damping: Float = 0.5,
    copy: bool = True,
    verbose: bool = False,
    return_n_iter: bool = False,
    random_state: RandomState | None | Int = None,
) -> tuple[ndarray, ndarray, int] | tuple[ndarray, ndarray]:
    ...


class AffinityPropagation(ClusterMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_iter_: int = ...
    affinity_matrix_: ndarray = ...
    labels_: ndarray = ...
    cluster_centers_: ndarray = ...
    cluster_centers_indices_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        damping: Float = 0.5,
        max_iter: Int = 200,
        convergence_iter: Int = 15,
        copy: bool = True,
        preference: float | None | ArrayLike = None,
        affinity: Literal["euclidean", "precomputed", "euclidean"] = "euclidean",
        verbose: bool = False,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def fit(
        self: AffinityPropagation_Self, X: MatrixLike, y: Any = None
    ) -> AffinityPropagation_Self:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def fit_predict(self, X: MatrixLike, y: Any = None) -> ndarray:
        ...



from .colors import Colormap, Normalize
from typing import Any, Callable, Literal, Sequence
from ._typing import *
from .transforms import Bbox, BboxBase, Transform
from .text import _AnnotationBase
from .backend_bases import Event, MouseEvent, RendererBase
from .patches import FancyBboxPatch
from .font_manager import FontProperties
from .figure import Figure
from .artist import Artist

DEBUG = ...

def bbox_artist(*args, **kwargs): ...

class OffsetBox(Artist):
    def __init__(self, *args, **kwargs) -> None: ...
    def set_figure(self, fig: Figure): ...
    def axes(self, ax): ...
    def contains(self, mouseevent: MouseEvent) -> tuple[bool, dict]: ...
    def set_offset(self, xy: Callable): ...
    def get_offset(
        self, width, height, xdescent, ydescent, renderer: RendererBase
    ) -> tuple[float, float]: ...
    def set_width(self, width: float): ...
    def set_height(self, height: float): ...
    def get_visible_children(self) -> list[Artist]: ...
    def get_children(self) -> list[Artist]: ...
    def get_extent_offsets(
        self, renderer: RendererBase
    ) -> tuple[float, float, float, float, list[tuple[float, float]]]: ...
    def get_extent(
        self, renderer: RendererBase
    ) -> tuple[float, float, float, float]: ...
    def get_window_extent(self, renderer: RendererBase = ...): ...
    def draw(self, renderer: RendererBase): ...

class PackerBase(OffsetBox):
    def __init__(
        self,
        pad: float = ...,
        sep: float = ...,
        width: float = ...,
        height: float = ...,
        align: Literal[
            "top", "bottom", "left", "right", "center", "baseline"
        ] = "baseline",
        mode: Literal["fixed", "expand", "equal"] = "fixed",
        children: list[Artist] = ...,
    ) -> None: ...

class VPacker(PackerBase):
    def get_extent_offsets(self, renderer: RendererBase) -> list: ...

class HPacker(PackerBase):
    def get_extent_offsets(self, renderer: RendererBase) -> list: ...

class PaddedBox(OffsetBox):
    def __init__(
        self,
        child: Artist,
        pad: float = ...,
        draw_frame: bool = ...,
        patch_attrs: dict | None = ...,
    ) -> None: ...
    def get_extent_offsets(self, renderer: RendererBase) -> list: ...
    def draw(self, renderer: RendererBase): ...
    def update_frame(self, bbox: Bbox, fontsize: float = ...): ...
    def draw_frame(self, renderer: RendererBase): ...

class DrawingArea(OffsetBox):
    def __init__(
        self,
        width: float,
        height: float,
        xdescent: float = ...,
        ydescent: float = ...,
        clip: bool = ...,
    ) -> None: ...
    @property
    def clip_children(self): ...
    @clip_children.setter
    def clip_children(self, val): ...
    def get_transform(self) -> Transform: ...
    def set_transform(self, t) -> None: ...
    def set_offset(self, xy: Sequence[float]) -> None: ...
    def get_offset(self) -> tuple[float, float]: ...
    def get_window_extent(self, renderer: RendererBase = ...): ...
    def get_extent(self, renderer: RendererBase): ...
    def add_artist(self, a: Artist): ...
    def draw(self, renderer: RendererBase): ...

class TextArea(OffsetBox):
    def __init__(
        self, s: str, textprops: dict = ..., multilinebaseline: bool = False
    ) -> None: ...
    def set_text(self, s: str): ...
    def get_text(self) -> str: ...
    def set_multilinebaseline(self, t: bool): ...
    def get_multilinebaseline(self) -> bool: ...
    def set_transform(self, t: Transform) -> None: ...
    def set_offset(self, xy: Sequence[float]) -> None: ...
    def get_offset(self) -> tuple[float, float]: ...
    def get_window_extent(self, renderer: RendererBase = ...): ...
    def get_extent(self, renderer: RendererBase): ...
    def draw(self, renderer: RendererBase): ...

class AuxTransformBox(OffsetBox):
    def __init__(self, aux_transform) -> None: ...
    def add_artist(self, a: Artist): ...
    def get_transform(self) -> Transform: ...
    def set_transform(self, t: Transform): ...
    def set_offset(self, xy: Sequence[float]): ...
    def get_offset(self) -> tuple[float, float]: ...
    def get_window_extent(self, renderer: RendererBase = ...): ...
    def get_extent(self, renderer: RendererBase): ...
    def draw(self, renderer: RendererBase): ...

class AnchoredOffsetbox(OffsetBox):

    zorder = ...
    codes = ...

    def __init__(
        self,
        loc: str,
        pad: float = ...,
        borderpad: float = ...,
        child: OffsetBox = ...,
        prop: FontProperties = ...,
        frameon: bool = ...,
        bbox_to_anchor: BboxBase | Sequence[float] = ...,
        bbox_transform: Transform | None = ...,
        **kwargs
    ) -> None: ...
    def set_child(self, child): ...
    def get_child(self): ...
    def get_children(self) -> list: ...
    def get_extent(self, renderer: RendererBase): ...
    def get_bbox_to_anchor(self) -> Bbox: ...
    def set_bbox_to_anchor(self, bbox: Bbox, transform: Transform = ...): ...
    def get_window_extent(self, renderer: RendererBase = ...): ...
    def update_frame(self, bbox: Bbox, fontsize: float = ...): ...
    def draw(self, renderer: RendererBase): ...

class AnchoredText(AnchoredOffsetbox):

    patch: FancyBboxPatch

    def __init__(
        self,
        s: str,
        loc: str,
        pad: float = ...,
        borderpad: float = ...,
        prop: dict = ...,
        **kwargs
    ) -> None: ...

class OffsetImage(OffsetBox):
    def __init__(
        self,
        arr,
        zoom: float = ...,
        cmap: Colormap = ...,
        norm: Normalize = ...,
        interpolation=...,
        origin=...,
        filternorm=...,
        filterrad=...,
        resample=...,
        dpi_cor=...,
        **kwargs
    ) -> None: ...
    def set_data(self, arr: ArrayLike): ...
    def get_data(self): ...
    def set_zoom(self, zoom: float): ...
    def get_zoom(self) -> float: ...
    def get_offset(self): ...
    def get_children(self): ...
    def get_window_extent(self, renderer: RendererBase = ...): ...
    def get_extent(self, renderer: RendererBase): ...
    def draw(self, renderer: RendererBase): ...

class AnnotationBbox(Artist, _AnnotationBase):

    zorder = ...
    def __str__(self) -> str: ...
    def __init__(
        self,
        offsetbox,
        xy: Sequence[float],
        xybox: Sequence[float] = ...,
        xycoords: str | Artist | Transform | Callable | Sequence[float] = "data",
        boxcoords: str | Artist | Transform | Callable | Sequence[float] = ...,
        frameon: bool = True,
        pad: float = 0.4,
        annotation_clip=...,
        box_alignment: Sequence[float] = ...,
        bboxprops=...,
        arrowprops=...,
        fontsize: float = ...,
        **kwargs
    ) -> None: ...
    @property
    def xyann(self): ...
    @xyann.setter
    def xyann(self, xyann): ...
    @property
    def anncoords(self): ...
    @anncoords.setter
    def anncoords(self, coords): ...
    def contains(self, mouseevent: MouseEvent): ...
    def get_children(self) -> list: ...
    def set_figure(self, fig: Figure): ...
    def set_fontsize(self, s: float = ...): ...
    def get_fontsize(self) -> float: ...
    def get_window_extent(self, renderer: RendererBase = ...): ...
    def get_tightbbox(self, renderer: RendererBase = ...) -> Bbox: ...
    def update_positions(self, renderee: RendererBase): ...
    def draw(self, renderer: RendererBase): ...

class DraggableBase:
    def __init__(self, ref_artist, use_blit: bool = ...) -> None: ...
    def on_motion(self, evt: Event): ...
    def on_pick(self, evt: Event): ...
    def on_release(self, event: Event): ...
    def disconnect(self): ...
    def save_offset(self): ...
    def update_offset(self, dx: float, dy: float): ...
    def finalize_offset(self): ...

class DraggableOffsetBox(DraggableBase):
    def __init__(self, ref_artist, offsetbox, use_blit: bool = ...) -> None: ...
    def save_offset(self): ...
    def update_offset(self, dx: float, dy: float): ...
    def get_loc_in_canvas(self): ...

class DraggableAnnotation(DraggableBase):
    def __init__(self, annotation, use_blit: bool = ...) -> None: ...
    def save_offset(self): ...
    def update_offset(self, dx: float, dy: float): ...



from vispy.util.svg.color import Color

from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import warnings

import numpy as np

from ..color import ColorArray
from ..gloo import VertexBuffer, _check_valid
from .shaders import Function, Variable
from .visual import Visual

_VERTEX_SHADER: str = ...

_FRAGMENT_SHADER: str = ...

disc: str = ...

arrow: str = ...

ring: str = ...

clobber: str = ...

square: str = ...

x_: str = ...

diamond: str = ...

vbar: str = ...

hbar: str = ...

cross: str = ...

tailed_arrow: str = ...

triangle_up: str = ...

triangle_down: str = ...

star: str = ...

# the following two markers needs x and y sizes
rect: str = ...

ellipse: str = ...

cross_lines: str = ...

class MarkersVisual(Visual):
    _marker_funcs: dict = ...

    _shaders: dict = ...

    def __init__(
        self,
        symbol: str = "o",
        scaling: bool = False,
        alpha: float = 1,
        antialias: float = 1,
        spherical: bool = False,
        light_color: ColorArray | Color | str = "white",
        light_position: ArrayLike = ...,
        light_ambient: float = 0.3,
        **kwargs,
    ): ...
    def set_data(
        self,
        pos: ArrayLike | None = None,
        size: float | ArrayLike = 10.0,
        edge_width: float | ArrayLike | None = 1.0,
        edge_width_rel: float | ArrayLike | None = None,
        edge_color: ColorArray | Color | str = "black",
        face_color: ColorArray | Color | str = "white",
        symbol=None,
        scaling=None,
    ): ...
    @property
    def symbols(self): ...
    @property
    def symbol(self): ...
    @symbol.setter
    def symbol(self, symbol): ...
    @property
    def scaling(self): ...
    @scaling.setter
    def scaling(self, value): ...
    @property
    def antialias(self): ...
    @antialias.setter
    def antialias(self, value): ...
    @property
    def light_position(self): ...
    @light_position.setter
    def light_position(self, value): ...
    @property
    def light_ambient(self): ...
    @light_ambient.setter
    def light_ambient(self, value): ...
    @property
    def light_color(self): ...
    @light_color.setter
    def light_color(self, value): ...
    @property
    def alpha(self): ...
    @alpha.setter
    def alpha(self, value): ...
    @property
    def spherical(self): ...
    @spherical.setter
    def spherical(self, value): ...
    def _prepare_transforms(self, view): ...
    def _prepare_draw(self, view): ...
    def _compute_bounds(self, axis, view): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.optimize._lbfgsb, version: $Revision: $
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__version__: bytes
def setulb(m, x, l, u, nbd, f, g, factr, pgtol, wa, iwa, task, iprint, csave, lsave, isave, dsave, maxls, n=...) -> typing.Any:
    "setulb(m,x,l,u,nbd,f,g,factr,pgtol,wa,iwa,task,iprint,csave,lsave,isave,dsave,maxls,[n])\n\nWrapper for ``setulb``.\n\nParameters\n----------\nm : input int\nx : in/output rank-1 array('d') with bounds (n)\nl : input rank-1 array('d') with bounds (n)\nu : input rank-1 array('d') with bounds (n)\nnbd : input rank-1 array('i') with bounds (n)\nf : in/output rank-0 array(float,'d')\ng : in/output rank-1 array('d') with bounds (n)\nfactr : input float\npgtol : input float\nwa : in/output rank-1 array('d') with bounds (2*m*n+5*n+11*m*m+8*m)\niwa : in/output rank-1 array('i') with bounds (3 * n)\ntask : in/output rank-0 array(string(len=60),'c')\niprint : input int\ncsave : in/output rank-0 array(string(len=60),'c')\nlsave : in/output rank-1 array('i') with bounds (4)\nisave : in/output rank-1 array('i') with bounds (44)\ndsave : in/output rank-1 array('d') with bounds (29)\nmaxls : input int\n\nOther Parameters\n----------------\nn : input int, optional\n    Default: len(x)\n"
    ...

def types() -> typing.Any:
    "'i'-scalar\n"
    ...

def __getattr__(name) -> typing.Any:
    ...




from typing import Any
from xml.sax.expatreader import ExpatParser as _ExpatParser  # type: ignore

from django.core.serializers import base as base

class Serializer(base.Serializer):
    def indent(self, level: Any) -> None: ...
    xml: Any = ...
    def start_serialization(self) -> None: ...
    def end_serialization(self) -> None: ...
    def start_object(self, obj: Any) -> None: ...
    def end_object(self, obj: Any) -> None: ...
    def handle_field(self, obj: Any, field: Any) -> None: ...
    def handle_fk_field(self, obj: Any, field: Any) -> None: ...
    def handle_m2m_field(self, obj: Any, field: Any) -> None: ...

class Deserializer(base.Deserializer):
    handle_forward_references: Any = ...
    event_stream: Any = ...
    db: Any = ...
    ignore: Any = ...
    def __init__(
        self,
        stream_or_string: Any,
        *,
        using: Any = ...,
        ignorenonexistent: bool = ...,
        **options: Any
    ) -> None: ...
    def __next__(self) -> Any: ...

def getInnerText(node: Any) -> Any: ...

class DefusedExpatParser(_ExpatParser):  # type: ignore [misc, no-any-unimported]
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def start_doctype_decl(
        self, name: Any, sysid: Any, pubid: Any, has_internal_subset: Any
    ) -> None: ...
    def entity_decl(
        self,
        name: Any,
        is_parameter_entity: Any,
        value: Any,
        base: Any,
        sysid: Any,
        pubid: Any,
        notation_name: Any,
    ) -> None: ...
    def unparsed_entity_decl(
        self, name: Any, base: Any, sysid: Any, pubid: Any, notation_name: Any
    ) -> None: ...
    def external_entity_ref_handler(
        self, context: Any, base: Any, sysid: Any, pubid: Any
    ) -> None: ...
    def reset(self) -> None: ...

class DefusedXmlException(ValueError): ...

class DTDForbidden(DefusedXmlException):
    name: Any = ...
    sysid: Any = ...
    pubid: Any = ...
    def __init__(self, name: Any, sysid: Any, pubid: Any) -> None: ...

class EntitiesForbidden(DefusedXmlException):
    name: Any = ...
    value: Any = ...
    base: Any = ...
    sysid: Any = ...
    pubid: Any = ...
    notation_name: Any = ...
    def __init__(
        self,
        name: Any,
        value: Any,
        base: Any,
        sysid: Any,
        pubid: Any,
        notation_name: Any,
    ) -> None: ...

class ExternalReferenceForbidden(DefusedXmlException):
    context: Any = ...
    base: Any = ...
    sysid: Any = ...
    pubid: Any = ...
    def __init__(self, context: Any, base: Any, sysid: Any, pubid: Any) -> None: ...



from ...algorithms.link_analysis.hits_alg import (
    hits as hits,
    hits_numpy as hits_numpy,
    hits_scipy as hits_scipy,
    authority_matrix as authority_matrix,
    hub_matrix as hub_matrix,
)
from ...algorithms.link_analysis.pagerank_alg import (
    pagerank as pagerank,
    pagerank_numpy as pagerank_numpy,
    pagerank_scipy as pagerank_scipy,
    google_matrix as google_matrix,
)



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.optimize._bglu_dense, version: unspecified
import typing
import builtins as _mod_builtins
import numpy.linalg as _mod_numpy_linalg

class BGLU(LU):
    '\n    Represents PLU factorization with Golub rank-one updates from\n    Bartels, Richard H. "A stabilization of the simplex method."\n    Numerische Mathematik 16.5 (1971): 414-434.\n    '
    @property
    def L(self) -> typing.Any:
        ...
    
    @property
    def U(self) -> typing.Any:
        ...
    
    def __init__(self) -> None:
        '\n        Given matrix A and basis indices b, perform PLU factorization of\n        basis matrix B\n        '
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    def __reduce__(self) -> typing.Union[str, typing.Tuple[typing.Any, ...]]:
        ...
    
    def __setstate__(self, state: typing.Any) -> None:
        ...
    
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    @property
    def average_solve_times(self) -> typing.Any:
        ...
    
    @property
    def bglu_time(self) -> typing.Any:
        ...
    
    @property
    def mast(self) -> typing.Any:
        ...
    
    @property
    def max_updates(self) -> typing.Any:
        ...
    
    @property
    def ops_list(self) -> typing.Any:
        ...
    
    def perform_perm(self) -> typing.Any:
        '\n        Perform individual row swaps defined in p returned by factor_lu to\n        generate final permutation indices pi\n        '
        ...
    
    @property
    def pi(self) -> typing.Any:
        ...
    
    @property
    def pit(self) -> typing.Any:
        ...
    
    @property
    def plu(self) -> typing.Any:
        ...
    
    def refactor(self, *args, **kwargs) -> typing.Any:
        ...
    
    def solve(self, *args, **kwargs) -> typing.Any:
        ...
    
    @property
    def solves(self) -> typing.Any:
        ...
    
    def update(self, *args, **kwargs) -> typing.Any:
        ...
    
    def update_basis(self) -> typing.Any:
        ...
    
    @property
    def updates(self) -> typing.Any:
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

class LU(_mod_builtins.object):
    '\n    Represents PLU factorization of a basis matrix with naive rank-one updates\n    '
    @property
    def A(self) -> typing.Any:
        ...
    
    @property
    def B(self) -> typing.Any:
        ...
    
    def __init__(self) -> None:
        ' Given matrix A and basis indices b, form basis matrix B '
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    def __reduce__(self) -> typing.Union[str, typing.Tuple[typing.Any, ...]]:
        ...
    
    def __setstate__(self, state: typing.Any) -> None:
        ...
    
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    @property
    def b(self) -> typing.Any:
        ...
    
    @property
    def m(self) -> typing.Any:
        ...
    
    @property
    def n(self) -> typing.Any:
        ...
    
    def solve(self) -> typing.Any:
        '\n        Solve B @ v = q\n        '
        ...
    
    def update(self) -> typing.Any:
        ' Rank-one update to basis and basis matrix '
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

LinAlgError = _mod_numpy_linalg.LinAlgError
__all__: list
__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_BGLU() -> typing.Any:
    ...

def __pyx_unpickle_Enum() -> typing.Any:
    ...

def __pyx_unpickle_LU() -> typing.Any:
    ...

__test__: dict
def _consider_refactor() -> typing.Any:
    '\n    This decorator records the time spent in the major BGLU\n    routines - refactor, update, and solve - in order to\n    calculate the average time required to solve a system.\n    It also forces PLU factorization of the basis matrix from\n    scratch to minimize the average solve time and to\n    accumulation of roundoff error.\n\n    Immediately after PLU factorization, the average solve time\n    will be rather high because PLU factorization is slow. For\n    some number of factor updates, the average solve time is\n    expected to decrease because the updates and solves are fast.\n    However, updates increase the compexity of the factorization,\n    so solve times are expected to increase with each update.\n    When the average solve time stops decreasing and begins\n    increasing, we perform PLU factorization from scratch rather\n    than updating. PLU factorization is also performed after the\n    maximum permitted number of updates is reached to prevent\n    further accumulation of roundoff error.\n    '
    ...

def lu_factor(a, overwrite_a, check_finite) -> typing.Any:
    "\n    Compute pivoted LU decomposition of a matrix.\n\n    The decomposition is::\n\n        A = P L U\n\n    where P is a permutation matrix, L lower triangular with unit\n    diagonal elements, and U upper triangular.\n\n    Parameters\n    ----------\n    a : (M, M) array_like\n        Matrix to decompose\n    overwrite_a : bool, optional\n        Whether to overwrite data in A (may increase performance)\n    check_finite : bool, optional\n        Whether to check that the input matrix contains only finite numbers.\n        Disabling may give a performance gain, but may result in problems\n        (crashes, non-termination) if the inputs do contain infinities or NaNs.\n\n    Returns\n    -------\n    lu : (N, N) ndarray\n        Matrix containing U in its upper triangle, and L in its lower triangle.\n        The unit diagonal elements of L are not stored.\n    piv : (N,) ndarray\n        Pivot indices representing the permutation matrix P:\n        row i of matrix was interchanged with row piv[i].\n\n    See also\n    --------\n    lu_solve : solve an equation system using the LU factorization of a matrix\n\n    Notes\n    -----\n    This is a wrapper to the ``*GETRF`` routines from LAPACK.\n\n    Examples\n    --------\n    >>> from scipy.linalg import lu_factor\n    >>> A = np.array([[2, 5, 8, 7], [5, 2, 2, 8], [7, 5, 6, 6], [5, 4, 4, 8]])\n    >>> lu, piv = lu_factor(A)\n    >>> piv\n    array([2, 2, 3, 3], dtype=int32)\n\n    Convert LAPACK's ``piv`` array to NumPy index and test the permutation\n\n    >>> piv_py = [2, 0, 3, 1]\n    >>> L, U = np.tril(lu, k=-1) + np.eye(4), np.triu(lu)\n    >>> np.allclose(A[piv_py] - L @ U, np.zeros((4, 4)))\n    True\n    "
    ...

def lu_solve(lu_and_piv, b, trans, overwrite_b, check_finite) -> typing.Any:
    'Solve an equation system, a x = b, given the LU factorization of a\n\n    Parameters\n    ----------\n    (lu, piv)\n        Factorization of the coefficient matrix a, as given by lu_factor\n    b : array\n        Right-hand side\n    trans : {0, 1, 2}, optional\n        Type of system to solve:\n\n        =====  =========\n        trans  system\n        =====  =========\n        0      a x   = b\n        1      a^T x = b\n        2      a^H x = b\n        =====  =========\n    overwrite_b : bool, optional\n        Whether to overwrite data in b (may increase performance)\n    check_finite : bool, optional\n        Whether to check that the input matrices contain only finite numbers.\n        Disabling may give a performance gain, but may result in problems\n        (crashes, non-termination) if the inputs do contain infinities or NaNs.\n\n    Returns\n    -------\n    x : array\n        Solution to the system\n\n    See also\n    --------\n    lu_factor : LU factorize a matrix\n\n    Examples\n    --------\n    >>> from scipy.linalg import lu_factor, lu_solve\n    >>> A = np.array([[2, 5, 8, 7], [5, 2, 2, 8], [7, 5, 6, 6], [5, 4, 4, 8]])\n    >>> b = np.array([1, 1, 1, 1])\n    >>> lu, piv = lu_factor(A)\n    >>> x = lu_solve((lu, piv), b)\n    >>> np.allclose(A @ x - b, np.zeros((4,)))\n    True\n\n    '
    ...

def solve(a, b, sym_pos, lower, overwrite_a, overwrite_b, debug, check_finite, assume_a, transposed) -> typing.Any:
    "\n    Solves the linear equation set ``a * x = b`` for the unknown ``x``\n    for square ``a`` matrix.\n\n    If the data matrix is known to be a particular type then supplying the\n    corresponding string to ``assume_a`` key chooses the dedicated solver.\n    The available options are\n\n    ===================  ========\n     generic matrix       'gen'\n     symmetric            'sym'\n     hermitian            'her'\n     positive definite    'pos'\n    ===================  ========\n\n    If omitted, ``'gen'`` is the default structure.\n\n    The datatype of the arrays define which solver is called regardless\n    of the values. In other words, even when the complex array entries have\n    precisely zero imaginary parts, the complex solver will be called based\n    on the data type of the array.\n\n    Parameters\n    ----------\n    a : (N, N) array_like\n        Square input data\n    b : (N, NRHS) array_like\n        Input data for the right hand side.\n    sym_pos : bool, optional\n        Assume `a` is symmetric and positive definite. This key is deprecated\n        and assume_a = 'pos' keyword is recommended instead. The functionality\n        is the same. It will be removed in the future.\n    lower : bool, optional\n        If True, only the data contained in the lower triangle of `a`. Default\n        is to use upper triangle. (ignored for ``'gen'``)\n    overwrite_a : bool, optional\n        Allow overwriting data in `a` (may enhance performance).\n        Default is False.\n    overwrite_b : bool, optional\n        Allow overwriting data in `b` (may enhance performance).\n        Default is False.\n    check_finite : bool, optional\n        Whether to check that the input matrices contain only finite numbers.\n        Disabling may give a performance gain, but may result in problems\n        (crashes, non-termination) if the inputs do contain infinities or NaNs.\n    assume_a : str, optional\n        Valid entries are explained above.\n    transposed: bool, optional\n        If True, ``a^T x = b`` for real matrices, raises `NotImplementedError`\n        for complex matrices (only for True).\n\n    Returns\n    -------\n    x : (N, NRHS) ndarray\n        The solution array.\n\n    Raises\n    ------\n    ValueError\n        If size mismatches detected or input a is not square.\n    LinAlgError\n        If the matrix is singular.\n    LinAlgWarning\n        If an ill-conditioned input a is detected.\n    NotImplementedError\n        If transposed is True and input a is a complex matrix.\n\n    Examples\n    --------\n    Given `a` and `b`, solve for `x`:\n\n    >>> a = np.array([[3, 2, 0], [1, -1, 0], [0, 5, 1]])\n    >>> b = np.array([2, 4, -1])\n    >>> from scipy import linalg\n    >>> x = linalg.solve(a, b)\n    >>> x\n    array([ 2., -2.,  9.])\n    >>> np.dot(a, x) == b\n    array([ True,  True,  True], dtype=bool)\n\n    Notes\n    -----\n    If the input b matrix is a 1-D array with N elements, when supplied\n    together with an NxN input a, it is assumed as a valid column vector\n    despite the apparent size mismatch. This is compatible with the\n    numpy.dot() behavior and the returned result is still 1-D array.\n\n    The generic, symmetric, Hermitian and positive definite solutions are\n    obtained via calling ?GESV, ?SYSV, ?HESV, and ?POSV routines of\n    LAPACK respectively.\n    "
    ...

def solve_triangular(a, b, trans, lower, unit_diagonal, overwrite_b, debug, check_finite) -> typing.Any:
    "\n    Solve the equation `a x = b` for `x`, assuming a is a triangular matrix.\n\n    Parameters\n    ----------\n    a : (M, M) array_like\n        A triangular matrix\n    b : (M,) or (M, N) array_like\n        Right-hand side matrix in `a x = b`\n    lower : bool, optional\n        Use only data contained in the lower triangle of `a`.\n        Default is to use upper triangle.\n    trans : {0, 1, 2, 'N', 'T', 'C'}, optional\n        Type of system to solve:\n\n        ========  =========\n        trans     system\n        ========  =========\n        0 or 'N'  a x  = b\n        1 or 'T'  a^T x = b\n        2 or 'C'  a^H x = b\n        ========  =========\n    unit_diagonal : bool, optional\n        If True, diagonal elements of `a` are assumed to be 1 and\n        will not be referenced.\n    overwrite_b : bool, optional\n        Allow overwriting data in `b` (may enhance performance)\n    check_finite : bool, optional\n        Whether to check that the input matrices contain only finite numbers.\n        Disabling may give a performance gain, but may result in problems\n        (crashes, non-termination) if the inputs do contain infinities or NaNs.\n\n    Returns\n    -------\n    x : (M,) or (M, N) ndarray\n        Solution to the system `a x = b`.  Shape of return matches `b`.\n\n    Raises\n    ------\n    LinAlgError\n        If `a` is singular\n\n    Notes\n    -----\n    .. versionadded:: 0.9.0\n\n    Examples\n    --------\n    Solve the lower triangular system a x = b, where::\n\n             [3  0  0  0]       [4]\n        a =  [2  1  0  0]   b = [2]\n             [1  0  1  0]       [4]\n             [1  1  1  1]       [2]\n\n    >>> from scipy.linalg import solve_triangular\n    >>> a = np.array([[3, 0, 0, 0], [2, 1, 0, 0], [1, 0, 1, 0], [1, 1, 1, 1]])\n    >>> b = np.array([4, 2, 4, 2])\n    >>> x = solve_triangular(a, b, lower=True)\n    >>> x\n    array([ 1.33333333, -0.66666667,  2.66666667, -1.33333333])\n    >>> a.dot(x)  # Check the result\n    array([ 4.,  2.,  4.,  2.])\n\n    "
    ...

def timer() -> float:
    'process_time() -> float\n\nProcess time for profiling: sum of the kernel and user-space CPU time.'
    ...

def __getattr__(name) -> typing.Any:
    ...




"""
This type stub file was generated by pyright.
"""

"""
Utilities for dealing with text encodings
"""
def get_stream_enc(stream, default=...): # -> None:
    """Return the given stream's encoding or a default.

    There are cases where ``sys.std*`` might not actually be a stream, so
    check for the encoding attribute prior to returning it, and return
    a default if it doesn't exist or evaluates as False. ``default``
    is None if not provided.
    """
    ...

def getdefaultencoding(prefer_stream=...):
    """Return IPython's guess for the default encoding for bytes as text.

    If prefer_stream is True (default), asks for stdin.encoding first,
    to match the calling Terminal, but that is often None for subprocesses.

    Then fall back on locale.getpreferredencoding(),
    which should be a sensible platform default (that respects LANG environment),
    and finally to sys.getdefaultencoding() which is the most conservative option,
    and usually UTF8 as of Python 3.
    """
    ...

DEFAULT_ENCODING = ...



from typing import Mapping
from .flow_matrix import (
    CGInverseLaplacian,
    FullInverseLaplacian,
    SuperLUInverseLaplacian,
    flow_matrix_row,
)
from ...classes.graph import Graph
from ...utils import (
    not_implemented_for,
    py_random_state,
    reverse_cuthill_mckee_ordering,
)

__all__ = [
    "current_flow_betweenness_centrality",
    "approximate_current_flow_betweenness_centrality",
    "edge_current_flow_betweenness_centrality",
]

@py_random_state(7)
def approximate_current_flow_betweenness_centrality(
    G: Graph,
    normalized=True,
    weight=None,
    dtype=...,
    solver="full",
    epsilon: float = 0.5,
    kmax: int = 10000,
    seed=None,
) -> Mapping: ...
def current_flow_betweenness_centrality(
    G: Graph, normalized=True, weight=None, dtype=..., solver="full"
) -> Mapping: ...
def edge_current_flow_betweenness_centrality(
    G: Graph, normalized=True, weight=None, dtype=..., solver="full"
) -> Mapping: ...



from typing import Any, ClassVar, Literal, TypeVar
from numpy.random import RandomState
from scipy.sparse.linalg import eigsh as eigsh, LinearOperator
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from scipy.linalg import svd as svd, qr as qr, solve as solve
from numbers import Integral as Integral, Real as Real
from ..utils.extmath import stable_cumsum as stable_cumsum
from ..neighbors._unsupervised import NearestNeighbors
from scipy.sparse import eye as eye
from ..base import (
    BaseEstimator,
    TransformerMixin,
    _UnstableArchMixin,
    ClassNamePrefixFeaturesOutMixin,
)
from scipy.sparse._csr import csr_matrix
from .._typing import MatrixLike, Float, ArrayLike, Int
from ..utils import check_random_state as check_random_state, check_array as check_array
from ..utils.validation import (
    check_is_fitted as check_is_fitted,
    FLOAT_DTYPES as FLOAT_DTYPES,
)

LocallyLinearEmbedding_Self = TypeVar(
    "LocallyLinearEmbedding_Self", bound="LocallyLinearEmbedding"
)


import numpy as np


def barycenter_weights(
    X: MatrixLike, Y: MatrixLike, indices: MatrixLike, reg: Float = 1e-3
) -> ndarray:
    ...


def barycenter_kneighbors_graph(
    X: ArrayLike | NearestNeighbors,
    n_neighbors: Int,
    reg: Float = 1e-3,
    n_jobs: None | int = None,
) -> csr_matrix:
    ...


def null_space(
    M: MatrixLike | LinearOperator,
    k: Int,
    k_skip: Int = 1,
    eigen_solver: Literal["auto", "arpack", "dense", "arpack"] = "arpack",
    tol: Float = 1e-6,
    max_iter: Int = 100,
    random_state: None | Int | RandomState = None,
) -> tuple[ndarray, Float]:
    ...


def locally_linear_embedding(
    X: ArrayLike | NearestNeighbors,
    *,
    n_neighbors: Int,
    n_components: Int,
    reg: Float = 1e-3,
    eigen_solver: Literal["auto", "arpack", "dense", "auto"] = "auto",
    tol: Float = 1e-6,
    max_iter: Int = 100,
    method: Literal["standard", "hessian", "modified", "ltsa", "standard"] = "standard",
    hessian_tol: Float = 1e-4,
    modified_tol: Float = 1e-12,
    random_state: RandomState | None | Int = None,
    n_jobs: None | int = None,
) -> tuple[ndarray, Float] | tuple[ndarray, float]:
    ...


class LocallyLinearEmbedding(
    ClassNamePrefixFeaturesOutMixin,
    TransformerMixin,
    _UnstableArchMixin,
    BaseEstimator,
):
    nbrs_: NearestNeighbors = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    reconstruction_error_: float = ...
    embedding_: ArrayLike = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        n_neighbors: Int = 5,
        n_components: Int = 2,
        reg: Float = 1e-3,
        eigen_solver: Literal["auto", "arpack", "dense", "auto"] = "auto",
        tol: Float = 1e-6,
        max_iter: Int = 100,
        method: Literal[
            "standard", "hessian", "modified", "ltsa", "standard"
        ] = "standard",
        hessian_tol: Float = 1e-4,
        modified_tol: Float = 1e-12,
        neighbors_algorithm: Literal[
            "auto", "brute", "kd_tree", "ball_tree", "auto"
        ] = "auto",
        random_state: RandomState | None | Int = None,
        n_jobs: None | int = None,
    ) -> None:
        ...

    def fit(
        self: LocallyLinearEmbedding_Self, X: MatrixLike, y: Any = None
    ) -> LocallyLinearEmbedding_Self:
        ...

    def fit_transform(self, X: MatrixLike, y: Any = None) -> ndarray:
        ...

    def transform(self, X: MatrixLike) -> ndarray:
        ...



from .maxflow import (
    maximum_flow as maximum_flow,
    maximum_flow_value as maximum_flow_value,
    minimum_cut as minimum_cut,
    minimum_cut_value as minimum_cut_value,
)
from .mincost import (
    min_cost_flow_cost as min_cost_flow_cost,
    min_cost_flow as min_cost_flow,
    cost_of_flow as cost_of_flow,
    max_flow_min_cost as max_flow_min_cost,
)
from .boykovkolmogorov import boykov_kolmogorov as boykov_kolmogorov
from .dinitz_alg import dinitz as dinitz
from .edmondskarp import edmonds_karp as edmonds_karp
from .gomory_hu import gomory_hu_tree as gomory_hu_tree
from .preflowpush import preflow_push as preflow_push
from .shortestaugmentingpath import shortest_augmenting_path as shortest_augmenting_path
from .capacityscaling import capacity_scaling as capacity_scaling
from .networksimplex import network_simplex as network_simplex
from .utils import (
    build_flow_dict as build_flow_dict,
    build_residual_network as build_residual_network,
)



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from .line import LineVisual as LineVisual  # noqa
from .arrow import ArrowVisual as ArrowVisual  # noqa



from typing import Any, ClassVar, Literal, Mapping, TypeVar
from numpy.random import RandomState
from ..base import BaseEstimator
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from ..utils.extmath import safe_sparse_dot as safe_sparse_dot
from scipy.sparse._csr import csr_matrix
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from ..utils.validation import check_is_fitted as check_is_fitted
from abc import ABCMeta, abstractmethod
from numpy import ndarray
from ..utils._param_validation import (
    Interval as Interval,
    StrOptions as StrOptions,
    Hidden as Hidden,
)
from numbers import Integral as Integral, Real as Real
from ._sgd_fast import (
    Hinge as Hinge,
    SquaredHinge as SquaredHinge,
    Log as Log,
    ModifiedHuber as ModifiedHuber,
    SquaredLoss as SquaredLoss,
    Huber as Huber,
    EpsilonInsensitive as EpsilonInsensitive,
    SquaredEpsilonInsensitive as SquaredEpsilonInsensitive,
)
from ..base import (
    clone as clone,
    is_classifier as is_classifier,
    RegressorMixin,
    OutlierMixin,
)
from ..model_selection import (
    StratifiedShuffleSplit as StratifiedShuffleSplit,
    ShuffleSplit as ShuffleSplit,
)
from ..utils import (
    check_random_state as check_random_state,
    compute_class_weight as compute_class_weight,
)
from ._base import LinearClassifierMixin, SparseCoefMixin, make_dataset as make_dataset
from ..utils.metaestimators import available_if as available_if
from .._typing import Float, Int, MatrixLike, ArrayLike

BaseSGDClassifier_Self = TypeVar("BaseSGDClassifier_Self", bound="BaseSGDClassifier")
SGDOneClassSVM_Self = TypeVar("SGDOneClassSVM_Self", bound="SGDOneClassSVM")
BaseSGDRegressor_Self = TypeVar("BaseSGDRegressor_Self", bound="BaseSGDRegressor")

# Authors: Peter Prettenhofer <peter.prettenhofer@gmail.com> (main author)
#          Mathieu Blondel (partial_fit support)
#
# License: BSD 3 clause

import numpy as np
import warnings

LEARNING_RATE_TYPES: dict = ...

PENALTY_TYPES: dict = ...

DEFAULT_EPSILON: float = ...
# Default value of ``epsilon`` parameter.

MAX_INT = ...


class _ValidationScoreCallback:
    def __init__(
        self,
        estimator: SGDClassifier,
        X_val: csr_matrix | ndarray,
        y_val: ndarray,
        sample_weight_val: ndarray,
        classes: None | ndarray = None,
    ) -> None:
        ...

    def __call__(self, coef: ndarray, intercept: float) -> Float:
        ...


class BaseSGD(SparseCoefMixin, BaseEstimator, metaclass=ABCMeta):

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        loss: str,
        *,
        penalty: str = "l2",
        alpha: float = 0.0001,
        C: float = 1.0,
        l1_ratio: float = 0.15,
        fit_intercept: bool = True,
        max_iter: int = 1000,
        tol: float = 1e-3,
        shuffle: bool = True,
        verbose: int = 0,
        epsilon: float = 0.1,
        random_state=None,
        learning_rate: str = "optimal",
        eta0: float = 0.0,
        power_t: float = 0.5,
        early_stopping: bool = False,
        validation_fraction: float = 0.1,
        n_iter_no_change: int = 5,
        warm_start: bool = False,
        average: bool = False,
    ) -> None:
        ...

    @abstractmethod
    def fit(self, X, y):
        ...


def fit_binary(
    est: BaseEstimator,
    i: Int,
    X: MatrixLike,
    y: ArrayLike,
    alpha: Float,
    C: Float,
    learning_rate: str,
    max_iter: Int,
    pos_weight: Float,
    neg_weight: Float,
    sample_weight: ArrayLike,
    validation_mask: None | ArrayLike = None,
    random_state: RandomState | None | Int = None,
) -> tuple[ndarray, float, int]:
    ...


class BaseSGDClassifier(LinearClassifierMixin, BaseSGD, metaclass=ABCMeta):

    # TODO(1.3): Remove "log""
    loss_functions: ClassVar[dict] = ...

    _parameter_constraints: ClassVar[dict] = ...

    @abstractmethod
    def __init__(
        self,
        loss: str = "hinge",
        *,
        penalty: str = "l2",
        alpha: float = 0.0001,
        l1_ratio: float = 0.15,
        fit_intercept: bool = True,
        max_iter: int = 1000,
        tol: float = 1e-3,
        shuffle: bool = True,
        verbose: int = 0,
        epsilon=...,
        n_jobs=None,
        random_state=None,
        learning_rate: str = "optimal",
        eta0: float = 0.0,
        power_t: float = 0.5,
        early_stopping: bool = False,
        validation_fraction: float = 0.1,
        n_iter_no_change: int = 5,
        class_weight=None,
        warm_start: bool = False,
        average: bool = False,
    ) -> None:
        ...

    def partial_fit(
        self: BaseSGDClassifier_Self,
        X: MatrixLike,
        y: ArrayLike,
        classes: None | ArrayLike = None,
        sample_weight: None | ArrayLike = None,
    ) -> BaseSGDClassifier_Self:
        ...

    def fit(
        self: BaseSGDClassifier_Self,
        X: MatrixLike,
        y: ArrayLike,
        coef_init: None | MatrixLike = None,
        intercept_init: None | ArrayLike = None,
        sample_weight: None | ArrayLike = None,
    ) -> BaseSGDClassifier_Self:
        ...


class SGDClassifier(BaseSGDClassifier):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    t_: int = ...
    classes_: ndarray = ...
    loss_function_: LossFunction = ...
    n_iter_: int = ...
    intercept_: ndarray = ...
    coef_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        loss: Literal[
            "hinge",
            "log_loss",
            "log",
            "modified_huber",
            "squared_hinge",
            "perceptron",
            "squared_error",
            "huber",
            "epsilon_insensitive",
            "squared_epsilon_insensitive",
            "hinge",
        ] = "hinge",
        *,
        penalty: None | Literal["l2", "l1", "elasticnet", "l2"] = "l2",
        alpha: Float = 0.0001,
        l1_ratio: Float = 0.15,
        fit_intercept: bool = True,
        max_iter: Int = 1000,
        tol: None | Float = 1e-3,
        shuffle: bool = True,
        verbose: Int = 0,
        epsilon: Float = ...,
        n_jobs: None | Int = None,
        random_state: RandomState | None | Int = None,
        learning_rate: str = "optimal",
        eta0: Float = 0.0,
        power_t: Float = 0.5,
        early_stopping: bool = False,
        validation_fraction: Float = 0.1,
        n_iter_no_change: Int = 5,
        class_weight: Mapping[str, float] | None | str = None,
        warm_start: bool = False,
        average: int | bool = False,
    ) -> None:
        ...

    def predict_proba(self, X: MatrixLike) -> ndarray:
        ...

    def predict_log_proba(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...


class BaseSGDRegressor(RegressorMixin, BaseSGD):

    loss_functions: ClassVar[dict] = ...

    _parameter_constraints: ClassVar[dict] = ...

    @abstractmethod
    def __init__(
        self,
        loss: str = "squared_error",
        *,
        penalty: str = "l2",
        alpha: float = 0.0001,
        l1_ratio: float = 0.15,
        fit_intercept: bool = True,
        max_iter: int = 1000,
        tol: float = 1e-3,
        shuffle: bool = True,
        verbose: int = 0,
        epsilon=...,
        random_state=None,
        learning_rate: str = "invscaling",
        eta0: float = 0.01,
        power_t: float = 0.25,
        early_stopping: bool = False,
        validation_fraction: float = 0.1,
        n_iter_no_change: int = 5,
        warm_start: bool = False,
        average: bool = False,
    ) -> None:
        ...

    def partial_fit(
        self: BaseSGDRegressor_Self,
        X: MatrixLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> BaseSGDRegressor_Self:
        ...

    def fit(
        self: BaseSGDRegressor_Self,
        X: MatrixLike,
        y: ArrayLike,
        coef_init: None | ArrayLike = None,
        intercept_init: None | ArrayLike = None,
        sample_weight: None | ArrayLike = None,
    ) -> SGDRegressor | BaseSGDRegressor_Self:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...


class SGDRegressor(BaseSGDRegressor):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    t_: int = ...
    n_iter_: int = ...
    intercept_: ndarray = ...
    coef_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        loss: str = "squared_error",
        *,
        penalty: None | Literal["l2", "l1", "elasticnet", "l2"] = "l2",
        alpha: Float = 0.0001,
        l1_ratio: Float = 0.15,
        fit_intercept: bool = True,
        max_iter: Int = 1000,
        tol: None | Float = 1e-3,
        shuffle: bool = True,
        verbose: Int = 0,
        epsilon: Float = ...,
        random_state: RandomState | None | Int = None,
        learning_rate: str = "invscaling",
        eta0: Float = 0.01,
        power_t: Float = 0.25,
        early_stopping: bool = False,
        validation_fraction: Float = 0.1,
        n_iter_no_change: Int = 5,
        warm_start: bool = False,
        average: int | bool = False,
    ) -> None:
        ...


class SGDOneClassSVM(BaseSGD, OutlierMixin):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    loss_function_: LossFunction = ...
    t_: int = ...
    n_iter_: int = ...
    offset_: ndarray = ...
    coef_: ndarray = ...

    loss_functions: ClassVar[dict] = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        nu: Float = 0.5,
        fit_intercept: bool = True,
        max_iter: Int = 1000,
        tol: None | Float = 1e-3,
        shuffle: bool = True,
        verbose: Int = 0,
        random_state: RandomState | None | Int = None,
        learning_rate: Literal[
            "optimal", "constant", "optimal", "invscaling", "adaptive"
        ] = "optimal",
        eta0: Float = 0.0,
        power_t: Float = 0.5,
        warm_start: bool = False,
        average: int | bool = False,
    ) -> None:
        ...

    def partial_fit(
        self: SGDOneClassSVM_Self,
        X: MatrixLike,
        y: Any = None,
        sample_weight: None | ArrayLike = None,
    ) -> SGDOneClassSVM_Self:
        ...

    def fit(
        self: SGDOneClassSVM_Self,
        X: MatrixLike,
        y: Any = None,
        coef_init: None | MatrixLike = None,
        offset_init: None | ArrayLike = None,
        sample_weight: None | ArrayLike = None,
    ) -> SGDOneClassSVM_Self:
        ...

    def decision_function(self, X: MatrixLike) -> ndarray:
        ...

    def score_samples(self, X: MatrixLike) -> ndarray:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...



from collections import defaultdict
from ...classes.graph import Graph

__all__ = ["equitable_color"]

def is_coloring(G: Graph, coloring): ...
def is_equitable(G: Graph, coloring, num_colors=None): ...
def make_C_from_F(F): ...
def make_N_from_L_C(L, C): ...
def make_H_from_C_N(C, N): ...
def change_color(u, X, Y, N, H, F, C, L): ...
def move_witnesses(src_color, dst_color, N, H, F, C, T_cal, L): ...
def pad_graph(G: Graph, num_colors): ...
def procedure_P(V_minus, V_plus, N, H, F, C, L, excluded_colors=None): ...
def equitable_color(G: Graph, num_colors): ...



from numpy.typing import ArrayLike
from ...classes.graph import Graph
from ...utils.decorators import not_implemented_for

__all__ = ["local_and_global_consistency"]

def local_and_global_consistency(
    G: Graph, alpha: float = 0.99, max_iter: int = 30, label_name: str = "label"
) -> ArrayLike: ...



from typing import ClassVar, Literal, Type
from ..utils._readonly_array_wrapper import ReadonlyArrayWrapper as ReadonlyArrayWrapper
from scipy.special import xlogy as xlogy
from numpy import ndarray
from .link import (
    Interval,
    IdentityLink,
    LogLink as LogLink,
    LogitLink as LogitLink,
    MultinomialLogit as MultinomialLogit,
)
from .link import BaseLink
from ._loss import (
    CyHalfSquaredError,
    CyAbsoluteError as CyAbsoluteError,
    CyPinballLoss as CyPinballLoss,
    CyHalfPoissonLoss as CyHalfPoissonLoss,
    CyHalfGammaLoss as CyHalfGammaLoss,
    CyHalfTweedieLoss as CyHalfTweedieLoss,
    CyHalfTweedieLossIdentity as CyHalfTweedieLossIdentity,
    CyHalfBinomialLoss as CyHalfBinomialLoss,
    CyHalfMultinomialLoss as CyHalfMultinomialLoss,
)
from .._typing import ArrayLike, MatrixLike, Int, Float, Scalar
from ..utils import check_scalar as check_scalar
from ._loss import CyLossFunction

# Goals:
# - Provide a common private module for loss functions/classes.
# - To be used in:
#   - LogisticRegression
#   - PoissonRegressor, GammaRegressor, TweedieRegressor
#   - HistGradientBoostingRegressor, HistGradientBoostingClassifier
#   - GradientBoostingRegressor, GradientBoostingClassifier
#   - SGDRegressor, SGDClassifier
# - Replace link module of GLMs.

import numbers
import numpy as np


# Note: The shape of raw_prediction for multiclass classifications are
# - GradientBoostingClassifier: (n_samples, n_classes)
# - HistGradientBoostingClassifier: (n_classes, n_samples)
#
# Note: Instead of inheritance like
#
#    class BaseLoss(BaseLink, CyLossFunction):
#    ...
#
#    # Note: Naturally, we would inherit in the following order
#    #     class HalfSquaredError(IdentityLink, CyHalfSquaredError, BaseLoss)
#    #   But because of https://github.com/cython/cython/issues/4350 we set BaseLoss as
#    #   the last one. This, of course, changes the MRO.
#    class HalfSquaredError(IdentityLink, CyHalfSquaredError, BaseLoss):
#
# we use composition. This way we improve maintainability by avoiding the above
# mentioned Cython edge case and have easier to understand code (which method calls
# which code).
class BaseLoss:
    constant_hessian: bool = ...
    approx_hessian: bool = ...
    interval_y_pred: Interval = ...
    interval_y_true: Interval = ...
    link: BaseLink = ...
    closs: CyLossFunction = ...

    # For decision trees:
    # This variable indicates whether the loss requires the leaves values to
    # be updated once the tree has been trained. The trees are trained to
    # predict a Newton-Raphson step (see grower._finalize_leaf()). But for
    # some losses (e.g. least absolute deviation) we need to adjust the tree
    # values to account for the "line search" of the gradient descent
    # procedure. See the original paper Greedy Function Approximation: A
    # Gradient Boosting Machine by Friedman
    # (https://statweb.stanford.edu/~jhf/ftp/trebst.pdf) for the theory.
    need_update_leaves_values: ClassVar[bool] = ...
    differentiable: ClassVar[bool] = ...
    is_multiclass: ClassVar[bool] = ...

    def __init__(self, closs, link, n_classes: None | int = None) -> None:
        ...

    def in_y_true_range(self, y: ArrayLike) -> bool:
        ...

    def in_y_pred_range(self, y: ArrayLike):
        ...

    def loss(
        self,
        y_true: ArrayLike,
        raw_prediction: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
        loss_out: None | ArrayLike = None,
        n_threads: Int = 1,
    ) -> ndarray:
        ...

    def loss_gradient(
        self,
        y_true: ArrayLike,
        raw_prediction: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
        loss_out: None | ArrayLike = None,
        gradient_out: None | MatrixLike | ArrayLike = None,
        n_threads: Int = 1,
    ) -> tuple[ndarray, ndarray]:
        ...

    def gradient(
        self,
        y_true: ArrayLike,
        raw_prediction: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
        gradient_out: None | MatrixLike | ArrayLike = None,
        n_threads: Int = 1,
    ) -> ndarray:
        ...

    def gradient_hessian(
        self,
        y_true: ArrayLike,
        raw_prediction: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
        gradient_out: None | MatrixLike | ArrayLike = None,
        hessian_out: None | MatrixLike | ArrayLike = None,
        n_threads: Int = 1,
    ) -> tuple[ndarray, ndarray]:
        ...

    def __call__(
        self,
        y_true: ArrayLike,
        raw_prediction: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
        n_threads: Int = 1,
    ) -> Float:
        ...

    def fit_intercept_only(
        self, y_true: ArrayLike, sample_weight: None | ArrayLike = None
    ) -> ndarray | Scalar | Float:
        ...

    def constant_to_optimal_zero(self, y_true, sample_weight=None):
        ...

    def init_gradient_and_hessian(
        self,
        n_samples: Int,
        dtype: Type[Float] | Float = ...,
        order: Literal["C", "F", "F"] = "F",
    ) -> tuple[ndarray, ndarray]:
        ...


# Note: Naturally, we would inherit in the following order
#         class HalfSquaredError(IdentityLink, CyHalfSquaredError, BaseLoss)
#       But because of https://github.com/cython/cython/issues/4350 we
#       set BaseLoss as the last one. This, of course, changes the MRO.
class HalfSquaredError(BaseLoss):
    def __init__(self, sample_weight=None) -> None:
        ...


class AbsoluteError(BaseLoss):

    differentiable: ClassVar[bool] = ...
    need_update_leaves_values: ClassVar[bool] = ...

    def __init__(self, sample_weight=None) -> None:
        ...

    def fit_intercept_only(self, y_true, sample_weight=None):
        ...


class PinballLoss(BaseLoss):

    differentiable: ClassVar[bool] = ...
    need_update_leaves_values: ClassVar[bool] = ...

    def __init__(self, sample_weight=None, quantile: float = 0.5) -> None:
        ...

    def fit_intercept_only(self, y_true: ndarray, sample_weight=None) -> Float:
        ...


class HalfPoissonLoss(BaseLoss):
    def __init__(self, sample_weight: None | ndarray = None) -> None:
        ...

    def constant_to_optimal_zero(self, y_true: ndarray, sample_weight=None) -> ndarray:
        ...


class HalfGammaLoss(BaseLoss):
    def __init__(self, sample_weight=None) -> None:
        ...

    def constant_to_optimal_zero(self, y_true: ndarray, sample_weight=None) -> ndarray:
        ...


class HalfTweedieLoss(BaseLoss):
    def __init__(self, sample_weight=None, power: float = 1.5) -> None:
        ...

    def constant_to_optimal_zero(self, y_true: ndarray, sample_weight=None) -> ndarray:
        ...


class HalfTweedieLossIdentity(BaseLoss):
    def __init__(self, sample_weight=None, power: float = 1.5) -> None:
        ...


class HalfBinomialLoss(BaseLoss):
    def __init__(self, sample_weight=None) -> None:
        ...

    def constant_to_optimal_zero(self, y_true, sample_weight=None):
        ...

    def predict_proba(self, raw_prediction: MatrixLike | ArrayLike) -> ndarray:
        ...


class HalfMultinomialLoss(BaseLoss):

    is_multiclass: ClassVar[bool] = ...

    def __init__(self, sample_weight=None, n_classes: int = 3) -> None:
        ...

    def in_y_true_range(self, y: ArrayLike):
        ...

    def fit_intercept_only(self, y_true, sample_weight=None):
        ...

    def predict_proba(self, raw_prediction: MatrixLike) -> ndarray:
        ...

    def gradient_proba(
        self,
        y_true: ArrayLike,
        raw_prediction: MatrixLike,
        sample_weight: None | ArrayLike = None,
        gradient_out: None | MatrixLike = None,
        proba_out: None | MatrixLike = None,
        n_threads: Int = 1,
    ) -> tuple[ndarray, ndarray]:
        ...


_LOSSES: dict = ...




def transform(raw_X, n_features: int, dtype,
              alternate_sign: bool|int=1, seed: int=0) -> tuple[int, list[int], list[int], list[float]]:
    """Guts of FeatureHasher.transform.
    Returns
    -------
    n_samples : integer
    indices, indptr, values : lists
        For constructing a scipy.sparse.csr_matrix.
    """
    ...



from numpy.typing import ArrayLike
from typing import Sequence
import math
import numpy as np
from scipy import spatial
import textwrap

from .._shared.utils import get_bound_method_class, safe_as_int

def _affine_matrix_from_vector(v): ...
def _center_and_normalize_points(points): ...
def _umeyama(src, dst, estimate_scale): ...

class GeometricTransform(object):
    def __call__(self, coords): ...
    def inverse(self, coords): ...
    def residuals(self, src, dst): ...
    def __add__(self, other): ...

class FundamentalMatrixTransform(GeometricTransform):
    def __init__(self, matrix=None, *, dimensionality=2): ...
    def __call__(self, coords): ...
    def inverse(self, coords): ...
    def _setup_constraint_matrix(self, src, dst): ...
    def estimate(self, src, dst) -> bool: ...
    def residuals(self, src, dst): ...

class EssentialMatrixTransform(FundamentalMatrixTransform):
    def __init__(
        self, rotation=None, translation=None, matrix=None, *, dimensionality=2
    ): ...
    def estimate(self, src, dst) -> bool: ...

class ProjectiveTransform(GeometricTransform):
    def __init__(self, matrix=None, *, dimensionality: int = 2): ...
    @property
    def _inv_matrix(self): ...
    def _apply_mat(self, coords, matrix): ...
    def __array__(self, dtype=None): ...
    def __call__(self, coords): ...
    def inverse(self, coords): ...
    def estimate(self, src, dst, weights=None) -> bool: ...
    def __add__(self, other): ...
    def __nice__(self): ...
    def __repr__(self): ...
    def __str__(self): ...
    @property
    def dimensionality(self): ...

class AffineTransform(ProjectiveTransform):
    def __init__(
        self,
        matrix=None,
        scale=None,
        rotation: float | None = None,
        shear: float | None = None,
        translation=None,
        *,
        dimensionality: int = 2
    ): ...
    @property
    def scale(self): ...
    @property
    def rotation(self): ...
    @property
    def shear(self): ...
    @property
    def translation(self): ...

class PiecewiseAffineTransform(GeometricTransform):
    def __init__(self): ...
    def estimate(self, src, dst) -> bool: ...
    def __call__(self, coords): ...
    def inverse(self, coords): ...

def _euler_rotation(axis, angle): ...
def _euler_rotation_matrix(angles, axes=None): ...

class EuclideanTransform(ProjectiveTransform):
    def __init__(
        self,
        matrix=None,
        rotation: float | Sequence[float] | None = None,
        translation=None,
        *,
        dimensionality: int = 2
    ): ...
    def estimate(self, src, dst) -> bool: ...
    @property
    def rotation(self): ...
    @property
    def translation(self): ...

class SimilarityTransform(EuclideanTransform):
    def __init__(
        self,
        matrix=None,
        scale: float | None = None,
        rotation: float | None = None,
        translation=None,
        *,
        dimensionality=2
    ): ...
    def estimate(self, src, dst) -> bool: ...
    @property
    def scale(self): ...

class PolynomialTransform(GeometricTransform):
    def __init__(self, params=None, *, dimensionality=2): ...
    def estimate(self, src, dst, order: int = 2, weights=None) -> bool: ...
    def __call__(self, coords): ...
    def inverse(self, coords): ...

TRANSFORMS: dict = ...

def estimate_transform(ttype, src, dst, *args, **kwargs: ArrayLike | int): ...
def matrix_transform(coords, matrix): ...



from numpy.typing import ArrayLike
from collections import Counter
from itertools import chain

from ...exception import NetworkXException

from ...classes.graph import Graph
from ...utils import not_implemented_for

__all__ = [
    "from_nested_tuple",
    "from_prufer_sequence",
    "NotATree",
    "to_nested_tuple",
    "to_prufer_sequence",
]

class NotATree(NetworkXException):
    pass

def to_nested_tuple(T, root, canonical_form: bool = False) -> tuple: ...
def from_nested_tuple(sequence: tuple, sensible_relabeling: bool = False): ...
def to_prufer_sequence(T) -> ArrayLike: ...
def from_prufer_sequence(sequence: ArrayLike): ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from .base_transform import BaseTransform as BaseTransform  # noqa
from .linear import (
    NullTransform as NullTransform,
    STTransform as STTransform,  # noqa
    MatrixTransform as MatrixTransform,
)  # noqa
from .nonlinear import (
    LogTransform as LogTransform,
    PolarTransform as PolarTransform,
)  # noqa
from .interactive import PanZoomTransform as PanZoomTransform
from .chain import ChainTransform as ChainTransform  # noqa
from ._util import (
    arg_to_array as arg_to_array,
    arg_to_vec4 as arg_to_vec4,
    as_vec4 as as_vec4,
    TransformCache as TransformCache,
)  # noqa
from .transform_system import TransformSystem as TransformSystem

__all__ = [
    "NullTransform",
    "STTransform",
    "MatrixTransform",
    "LogTransform",
    "PolarTransform",
    "ChainTransform",
    "TransformSystem",
    "PanZoomTransform",
]

transform_types: dict = ...

def create_transform(type, *args, **kwargs): ...



from typing import Any, ClassVar, TypeVar
from numpy.random import RandomState
from scipy import sparse as sparse
from itertools import product as product
from scipy.sparse._coo import coo_matrix
from numpy import ndarray
from ..utils._param_validation import Interval as Interval
from numbers import Integral as Integral, Number as Number, Real as Real
from numpy.lib.stride_tricks import as_strided as as_strided
from ..base import BaseEstimator
from scipy.sparse import spmatrix
from .._typing import MatrixLike, ArrayLike, Int
from ..utils import check_array as check_array, check_random_state as check_random_state

PatchExtractor_Self = TypeVar("PatchExtractor_Self", bound="PatchExtractor")

import numpy as np

__all__ = [
    "PatchExtractor",
    "extract_patches_2d",
    "grid_to_graph",
    "img_to_graph",
    "reconstruct_from_patches_2d",
]


def img_to_graph(
    img: MatrixLike,
    *,
    mask: None | MatrixLike = None,
    return_as: MatrixLike | ArrayLike = ...,
    dtype=None
) -> coo_matrix | ndarray:
    ...


def grid_to_graph(
    n_x: Int,
    n_y: Int,
    n_z: Int = 1,
    *,
    mask: None | MatrixLike = None,
    return_as: MatrixLike | ArrayLike = ...,
    dtype=...
) -> ndarray | spmatrix:
    ...


def extract_patches_2d(
    image: MatrixLike,
    patch_size: tuple[int, int],
    *,
    max_patches: float | None | int = None,
    random_state: RandomState | None | Int = None
) -> ndarray:
    ...


def reconstruct_from_patches_2d(
    patches: MatrixLike, image_size: tuple[int, int, int] | tuple[int, int]
) -> ndarray:
    ...


class PatchExtractor(BaseEstimator):

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        patch_size: None | tuple[int, int] = None,
        max_patches: float | None | int = None,
        random_state: RandomState | None | Int = None
    ) -> None:
        ...

    def fit(
        self: PatchExtractor_Self, X: MatrixLike, y: Any = None
    ) -> PatchExtractor_Self:
        ...

    def transform(self, X: MatrixLike) -> ndarray:
        ...



from typing import Any, ClassVar, Literal, Mapping, Sequence, TypeVar
from warnings import (
    catch_warnings as catch_warnings,
    simplefilter as simplefilter,
    warn as warn,
)
from numpy.random import RandomState
from ..tree._tree import DTYPE as DTYPE, DOUBLE as DOUBLE
from ..exceptions import DataConversionWarning as DataConversionWarning
from ..metrics import accuracy_score as accuracy_score, r2_score as r2_score
from ..preprocessing import OneHotEncoder
from scipy.sparse import issparse as issparse, hstack as sparse_hstack, spmatrix
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from ..utils.validation import check_is_fitted as check_is_fitted
from abc import ABCMeta, abstractmethod
from ..tree import (
    BaseDecisionTree as BaseDecisionTree,
    DecisionTreeClassifier,
    DecisionTreeRegressor,
    ExtraTreeClassifier as ExtraTreeClassifier,
    ExtraTreeRegressor,
)
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numbers import Integral as Integral, Real as Real
from ..base import (
    is_classifier as is_classifier,
    ClassifierMixin,
    MultiOutputMixin,
    RegressorMixin,
    TransformerMixin,
)
from ..utils import (
    check_random_state as check_random_state,
    compute_sample_weight as compute_sample_weight,
)
from ._base import BaseEnsemble
from ..utils.multiclass import (
    check_classification_targets as check_classification_targets,
    type_of_target as type_of_target,
)
from .._typing import MatrixLike, ArrayLike, Int, Float

BaseForest_Self = TypeVar("BaseForest_Self", bound="BaseForest")
RandomTreesEmbedding_Self = TypeVar(
    "RandomTreesEmbedding_Self", bound="RandomTreesEmbedding"
)

import threading
import numpy as np


__all__ = [
    "RandomForestClassifier",
    "RandomForestRegressor",
    "ExtraTreesClassifier",
    "ExtraTreesRegressor",
    "RandomTreesEmbedding",
]

MAX_INT = ...


class BaseForest(MultiOutputMixin, BaseEnsemble, metaclass=ABCMeta):

    _parameter_constraints: ClassVar[dict] = ...

    @abstractmethod
    def __init__(
        self,
        estimator,
        n_estimators: int = 100,
        *,
        estimator_params=...,
        bootstrap: bool = False,
        oob_score: bool = False,
        n_jobs=None,
        random_state=None,
        verbose: int = 0,
        warm_start: bool = False,
        class_weight=None,
        max_samples=None,
        base_estimator: str = "deprecated",
    ) -> None:
        ...

    def apply(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def decision_path(self, X: MatrixLike | ArrayLike) -> tuple[spmatrix, ndarray]:
        ...

    def fit(
        self: BaseForest_Self,
        X: MatrixLike | ArrayLike,
        y: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> BaseForest_Self:
        ...

    @property
    def feature_importances_(self) -> ndarray:
        ...


class ForestClassifier(ClassifierMixin, BaseForest, metaclass=ABCMeta):
    @abstractmethod
    def __init__(
        self,
        estimator,
        n_estimators: int = 100,
        *,
        estimator_params=...,
        bootstrap: bool = False,
        oob_score: bool = False,
        n_jobs=None,
        random_state=None,
        verbose: int = 0,
        warm_start: bool = False,
        class_weight=None,
        max_samples=None,
        base_estimator: str = "deprecated",
    ) -> None:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def predict_proba(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def predict_log_proba(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...


class ForestRegressor(RegressorMixin, BaseForest, metaclass=ABCMeta):
    @abstractmethod
    def __init__(
        self,
        estimator,
        n_estimators: int = 100,
        *,
        estimator_params=...,
        bootstrap: bool = False,
        oob_score: bool = False,
        n_jobs=None,
        random_state=None,
        verbose: int = 0,
        warm_start: bool = False,
        max_samples=None,
        base_estimator: str = "deprecated",
    ) -> None:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...


class RandomForestClassifier(ForestClassifier):
    oob_decision_function_: ndarray = ...
    oob_score_: float = ...
    feature_importances_: ndarray = ...
    n_outputs_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_classes_: list | int = ...
    classes_: ndarray = ...
    estimators_: list[DecisionTreeClassifier] = ...
    base_estimator_: DecisionTreeClassifier = ...
    estimator_: DecisionTreeClassifier = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_estimators: Int = 100,
        *,
        criterion: Literal["gini", "entropy", "log_loss", "gini"] = "gini",
        max_depth: None | Int = None,
        min_samples_split: float | int = 2,
        min_samples_leaf: float | int = 1,
        min_weight_fraction_leaf: Float = 0.0,
        max_features: float | Literal["sqrt", "log2", "sqrt"] | int = "sqrt",
        max_leaf_nodes: None | Int = None,
        min_impurity_decrease: Float = 0.0,
        bootstrap: bool = True,
        oob_score: bool = False,
        n_jobs: None | Int = None,
        random_state: RandomState | None | Int = None,
        verbose: Int = 0,
        warm_start: bool = False,
        class_weight: Literal["balanced", "balanced_subsample"]
        | None
        | Mapping
        | Sequence[Mapping] = None,
        ccp_alpha: float = 0.0,
        max_samples: float | None | int = None,
    ) -> None:
        ...


class RandomForestRegressor(ForestRegressor):
    oob_prediction_: ndarray = ...
    oob_score_: float = ...
    n_outputs_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    feature_importances_: ndarray = ...
    estimators_: list[DecisionTreeRegressor] = ...
    base_estimator_: DecisionTreeRegressor = ...
    estimator_: DecisionTreeRegressor = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_estimators: Int = 100,
        *,
        criterion: Literal[
            "squared_error",
            "absolute_error",
            "friedman_mse",
            "poisson",
            "squared_error",
        ] = "squared_error",
        max_depth: None | Int = None,
        min_samples_split: float | int = 2,
        min_samples_leaf: float | int = 1,
        min_weight_fraction_leaf: Float = 0.0,
        max_features: float | Literal["sqrt", "log2"] | int = 1.0,
        max_leaf_nodes: None | Int = None,
        min_impurity_decrease: Float = 0.0,
        bootstrap: bool = True,
        oob_score: bool = False,
        n_jobs: None | Int = None,
        random_state: RandomState | None | Int = None,
        verbose: Int = 0,
        warm_start: bool = False,
        ccp_alpha: float = 0.0,
        max_samples: float | None | int = None,
    ) -> None:
        ...


class ExtraTreesClassifier(ForestClassifier):
    oob_decision_function_: ndarray = ...
    oob_score_: float = ...
    n_outputs_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    feature_importances_: ndarray = ...
    n_classes_: list | int = ...
    classes_: ndarray = ...
    estimators_: list[DecisionTreeClassifier] = ...
    base_estimator_: ExtraTreesClassifier = ...
    estimator_: ExtraTreesClassifier = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_estimators: Int = 100,
        *,
        criterion: Literal["gini", "entropy", "log_loss", "gini"] = "gini",
        max_depth: None | Int = None,
        min_samples_split: float | int = 2,
        min_samples_leaf: float | int = 1,
        min_weight_fraction_leaf: Float = 0.0,
        max_features: float | Literal["sqrt", "log2", "sqrt"] | int = "sqrt",
        max_leaf_nodes: None | Int = None,
        min_impurity_decrease: Float = 0.0,
        bootstrap: bool = False,
        oob_score: bool = False,
        n_jobs: None | Int = None,
        random_state: RandomState | None | Int = None,
        verbose: Int = 0,
        warm_start: bool = False,
        class_weight: Literal["balanced", "balanced_subsample"]
        | None
        | Mapping
        | Sequence[Mapping] = None,
        ccp_alpha: float = 0.0,
        max_samples: float | None | int = None,
    ) -> None:
        ...


class ExtraTreesRegressor(ForestRegressor):
    oob_prediction_: ndarray = ...
    oob_score_: float = ...
    n_outputs_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    feature_importances_: ndarray = ...
    estimators_: list[DecisionTreeRegressor] = ...
    base_estimator_: ExtraTreeRegressor = ...
    estimator_: ExtraTreeRegressor = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_estimators: Int = 100,
        *,
        criterion: Literal[
            "squared_error",
            "absolute_error",
            "friedman_mse",
            "poisson",
            "squared_error",
        ] = "squared_error",
        max_depth: None | Int = None,
        min_samples_split: float | int = 2,
        min_samples_leaf: float | int = 1,
        min_weight_fraction_leaf: Float = 0.0,
        max_features: float | Literal["sqrt", "log2"] | int = 1.0,
        max_leaf_nodes: None | Int = None,
        min_impurity_decrease: Float = 0.0,
        bootstrap: bool = False,
        oob_score: bool = False,
        n_jobs: None | Int = None,
        random_state: RandomState | None | Int = None,
        verbose: Int = 0,
        warm_start: bool = False,
        ccp_alpha: float = 0.0,
        max_samples: float | None | int = None,
    ) -> None:
        ...


class RandomTreesEmbedding(TransformerMixin, BaseForest):
    one_hot_encoder_: OneHotEncoder = ...
    n_outputs_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    feature_importances_: ndarray = ...
    estimators_: list[ExtraTreeRegressor] = ...
    base_estimator_: ExtraTreeRegressor = ...
    estimator_: ExtraTreeRegressor = ...

    _parameter_constraints: ClassVar[dict] = ...
    for param in ("max_features", "ccp_alpha", "splitter"):
        pass

    criterion: ClassVar[str] = ...
    max_features: ClassVar[int] = ...

    def __init__(
        self,
        n_estimators: Int = 100,
        *,
        max_depth: Int = 5,
        min_samples_split: float | int = 2,
        min_samples_leaf: float | int = 1,
        min_weight_fraction_leaf: Float = 0.0,
        max_leaf_nodes: None | Int = None,
        min_impurity_decrease: Float = 0.0,
        sparse_output: bool = True,
        n_jobs: None | Int = None,
        random_state: RandomState | None | Int = None,
        verbose: Int = 0,
        warm_start: bool = False,
    ) -> None:
        ...

    def fit(
        self: RandomTreesEmbedding_Self,
        X: MatrixLike | ArrayLike,
        y: Any = None,
        sample_weight: None | ArrayLike = None,
    ) -> RandomTreesEmbedding_Self:
        ...

    def fit_transform(
        self,
        X: MatrixLike | ArrayLike,
        y: Any = None,
        sample_weight: None | ArrayLike = None,
    ) -> spmatrix:
        ...

    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...

    def transform(self, X: MatrixLike | ArrayLike) -> spmatrix:
        ...



from typing import Mapping
from ..classes.graph import Graph
from ..exception import NetworkXError
from ..utils import not_implemented_for

__all__ = [
    "core_number",
    "find_cores",
    "k_core",
    "k_shell",
    "k_crust",
    "k_corona",
    "k_truss",
    "onion_layers",
]

def core_number(G: Graph) -> Mapping: ...
def find_cores(G: Graph): ...
def k_core(G: Graph, k: int | None = None, core_number: Mapping | None = None): ...
def k_shell(G: Graph, k: int | None = None, core_number: Mapping | None = None): ...
def k_crust(G: Graph, k: int | None = None, core_number: Mapping | None = None): ...
def k_corona(G: Graph, k: int, core_number: Mapping | None = None): ...
def k_truss(G: Graph, k: int): ...
def onion_layers(G: Graph) -> Mapping: ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
__all__ = [
    "AxisWidget",
    "Console",
    "ColorBarWidget",
    "Grid",
    "Label",
    "ViewBox",
    "Widget",
]

from .console import Console as Console  # noqa
from .grid import Grid as Grid  # noqa
from .viewbox import ViewBox as ViewBox  # noqa
from .widget import Widget as Widget  # noqa
from .axis import AxisWidget as AxisWidget  # noqa
from .colorbar import ColorBarWidget as ColorBarWidget  # noqa
from .label import Label as Label  # noqa



from numpy import dtype
from numpy.typing import NDArray

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
import warnings

import numpy as np

from vispy.gloo import Texture2D, Texture3D
from vispy.gloo.texture import should_cast_to_f32

def get_default_clim_from_dtype(dtype): ...
def get_default_clim_from_data(data): ...

class _ScaledTextureMixin:
    def __init__(self, data: np.ndarray | tuple | None | None = None, **texture_kwargs): ...
    def init_scaling_texture(self, data=None, internalformat=None, **texture_kwargs): ...
    def _get_texture_format_for_data(self, data, internalformat): ...
    @property
    def clim(self): ...
    def set_clim(self, clim): ...
    @property
    def clim_normalized(self): ...
    @property
    def is_normalized(self): ...
    def normalize_value(self, val: NDArray | float | int, input_data_dtype: np.dtype): ...
    def _data_num_channels(self, data): ...
    def _create_rep_array(self, data): ...
    def check_data_format(self, data): ...
    def scale_and_set_data(self, data, offset=None, copy=False): ...

class CPUScaledTextureMixin(_ScaledTextureMixin):
    def __init__(self, data=None, **texture_kwargs): ...
    def _clim_outside_data_limits(self, cmin, cmax): ...
    def set_clim(self, clim): ...
    @property
    def clim_normalized(self): ...
    @staticmethod
    def _scale_data_on_cpu(data, clim, copy=True): ...
    def scale_and_set_data(self, data, offset=None, copy=True): ...

class GPUScaledTextureMixin(_ScaledTextureMixin):

    # dtype -> internalformat
    # 'r' will be replaced (if needed) with rgb or rgba depending on number of bands
    _texture_dtype_format: dict = ...
    # instance variable that will be used later on
    _auto_texture_format: bool = ...

    def _handle_auto_texture_format(self, texture_format, data): ...
    def _get_gl_tex_format(self, texture_format, num_channels): ...
    def _get_texture_format_for_data(self, data, internalformat): ...
    def _compute_clim(self, data): ...
    def _internalformat_will_change(self, data): ...
    def check_data_format(self, data): ...
    def _reformat_if_necessary(self, data): ...
    def scale_and_set_data(self, data, offset=None, copy=False): ...

class CPUScaledTexture2D(CPUScaledTextureMixin, Texture2D):
    pass

class GPUScaledTexture2D(GPUScaledTextureMixin, Texture2D):
    pass

class CPUScaledTexture3D(CPUScaledTextureMixin, Texture3D):
    pass

class GPUScaledTextured3D(GPUScaledTextureMixin, Texture3D):
    pass



from typing import Callable, ClassVar, Iterator, Literal, TypeVar
from numpy.random import RandomState
from ..tree._tree import DTYPE as DTYPE, DOUBLE as DOUBLE
from ..base import BaseEstimator
from ..exceptions import NotFittedError as NotFittedError
from ._gradient_boosting import (
    predict_stages as predict_stages,
    predict_stage as predict_stage,
)
from scipy.sparse import (
    csc_matrix as csc_matrix,
    csr_matrix as csr_matrix,
    issparse as issparse,
)
from ..utils.validation import check_is_fitted as check_is_fitted
from abc import ABCMeta, abstractmethod
from ..tree import DecisionTreeRegressor as DecisionTreeRegressor
from numpy import ndarray
from ..utils._param_validation import (
    HasMethods as HasMethods,
    Interval as Interval,
    StrOptions as StrOptions,
)
from numbers import Integral as Integral, Real as Real
from ..base import ClassifierMixin, RegressorMixin, is_classifier as is_classifier
from ..model_selection import train_test_split as train_test_split
from time import time as time
from ..utils import (
    deprecated,
    check_array as check_array,
    check_random_state as check_random_state,
    column_or_1d as column_or_1d,
)
from ._base import BaseEnsemble
from ..utils.multiclass import (
    check_classification_targets as check_classification_targets,
)
from ._gb_losses import LossFunction
from .._typing import Int, MatrixLike, ArrayLike, Float

BaseGradientBoosting_Self = TypeVar(
    "BaseGradientBoosting_Self", bound="BaseGradientBoosting"
)

import warnings

import numpy as np


class VerboseReporter:
    def __init__(self, verbose: Int) -> None:
        ...

    def init(self, est: BaseEstimator, begin_at_stage: Int = 0):
        ...

    def update(self, j: Int, est: BaseEstimator):
        ...


class BaseGradientBoosting(BaseEnsemble, metaclass=ABCMeta):

    _parameter_constraints: ClassVar[dict] = ...

    @abstractmethod
    def __init__(
        self,
        *,
        loss,
        learning_rate,
        n_estimators,
        criterion,
        min_samples_split,
        min_samples_leaf,
        min_weight_fraction_leaf,
        max_depth,
        min_impurity_decrease,
        init,
        subsample,
        max_features,
        ccp_alpha,
        random_state,
        alpha: float = 0.9,
        verbose: int = 0,
        max_leaf_nodes=None,
        warm_start: bool = False,
        validation_fraction: float = 0.1,
        n_iter_no_change=None,
        tol: float = 1e-4,
    ) -> None:
        ...

    def fit(
        self: BaseGradientBoosting_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
        monitor: None | Callable = None,
    ) -> BaseGradientBoosting_Self:
        ...

    @property
    def feature_importances_(self) -> ndarray:
        ...

    def apply(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    # TODO(1.3): Remove
    # mypy error: Decorated property not supported
    @deprecated(  # type: ignore
        "Attribute `loss_` was deprecated in version 1.1 and will be removed in 1.3."
    )
    def loss_(self):
        ...


class GradientBoostingClassifier(ClassifierMixin, BaseGradientBoosting):
    max_features_: int = ...
    n_classes_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    classes_: ndarray = ...
    estimators_: ndarray = ...
    init_: BaseEstimator = ...
    loss_: LossFunction = ...
    train_score_: ndarray = ...
    oob_improvement_: ndarray = ...
    feature_importances_: ndarray = ...
    n_estimators_: int = ...

    # TODO(1.3): remove "deviance"
    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        loss: Literal["log_loss", "deviance", "exponential", "log_loss"] = "log_loss",
        learning_rate: Float = 0.1,
        n_estimators: Int = 100,
        subsample: Float = 1.0,
        criterion: Literal[
            "friedman_mse", "squared_error", "friedman_mse"
        ] = "friedman_mse",
        min_samples_split: float | int = 2,
        min_samples_leaf: float | int = 1,
        min_weight_fraction_leaf: Float = 0.0,
        max_depth: None | int = 3,
        min_impurity_decrease: Float = 0.0,
        init: None | str | BaseEstimator = None,
        random_state: RandomState | None | Int = None,
        max_features: float | None | Literal["auto", "sqrt", "log2"] | int = None,
        verbose: Int = 0,
        max_leaf_nodes: None | Int = None,
        warm_start: bool = False,
        validation_fraction: Float = 0.1,
        n_iter_no_change: None | Int = None,
        tol: Float = 1e-4,
        ccp_alpha: float = 0.0,
    ) -> None:
        ...

    def decision_function(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def staged_decision_function(self, X: MatrixLike | ArrayLike):
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def staged_predict(self, X: MatrixLike | ArrayLike):
        ...

    def predict_proba(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def predict_log_proba(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def staged_predict_proba(self, X: MatrixLike | ArrayLike) -> Iterator[ndarray]:
        ...


class GradientBoostingRegressor(RegressorMixin, BaseGradientBoosting):
    max_features_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_estimators_: int = ...
    estimators_: ndarray = ...
    init_: BaseEstimator = ...
    loss_: LossFunction = ...
    train_score_: ndarray = ...
    oob_improvement_: ndarray = ...
    feature_importances_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        loss: Literal[
            "squared_error", "absolute_error", "huber", "quantile", "squared_error"
        ] = "squared_error",
        learning_rate: Float = 0.1,
        n_estimators: Int = 100,
        subsample: Float = 1.0,
        criterion: Literal[
            "friedman_mse", "squared_error", "friedman_mse"
        ] = "friedman_mse",
        min_samples_split: float | int = 2,
        min_samples_leaf: float | int = 1,
        min_weight_fraction_leaf: Float = 0.0,
        max_depth: None | int = 3,
        min_impurity_decrease: Float = 0.0,
        init: None | str | BaseEstimator = None,
        random_state: RandomState | None | Int = None,
        max_features: float | None | Literal["auto", "sqrt", "log2"] | int = None,
        alpha: Float = 0.9,
        verbose: Int = 0,
        max_leaf_nodes: None | Int = None,
        warm_start: bool = False,
        validation_fraction: Float = 0.1,
        n_iter_no_change: None | Int = None,
        tol: Float = 1e-4,
        ccp_alpha: float = 0.0,
    ) -> None:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def staged_predict(self, X: MatrixLike | ArrayLike) -> Iterator[ndarray]:
        ...

    def apply(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...



import copy

from ..classes.graph import Graph

__all__ = ["kl_connected_subgraph", "is_kl_connected"]

def kl_connected_subgraph(
    G: Graph, k, l, low_memory: bool = False, same_as_graph: bool = False
): ...
def is_kl_connected(G: Graph, k, l, low_memory: bool = False) -> bool: ...
